# django imports
from django.core.cache import cache
from django.contrib.auth.models import User
from django.db import models
from django.utils.translation import ugettext_lazy as _

# lfs imports
from lfs.catalog.models import Product

class Cart(models.Model):
    """A cart is a container for products which are supposed to be bought by a
    shop customer.

    Instance variables:

    - user
       The user to which the cart belongs to
    - session
       The session to which the cart belongs to

    A cart can be assigned either to the current logged in User (in case
    the shop user is logged in) or to the current session (in case the shop user
    is not logged in).

    A cart is only created if it needs to. When the shop user adds something to
    the cart.
    """
    user = models.ForeignKey(User, verbose_name=_(u"User"), blank=True, null=True)
    session = models.CharField(_(u"Session"), blank=True, max_length=100)
    creation_date = models.DateTimeField(_(u"Creation date"), auto_now_add=True)
    modification_date = models.DateTimeField(_(u"Modification date"), auto_now=True, auto_now_add=True)

    def items(self):
        """Returns the items of the cart.
        """
        cache_key = "cart-items-%s" % self.id
        items = cache.get(cache_key)
        if items is None:
            items = CartItem.objects.filter(cart=self)
            cache.set(cache_key, items)
        return items

    @property
    def amount_of_items(self):
        """Returns the amount of items of the cart.
        """
        amount = 0
        for item in self.items():
            amount += item.amount
        return amount

    def get_name(self):
        cart_name = ""
        for cart_item in self.items.all():
            if cart_name.product is not None:
                cart_name = cart_name + cart_name.product.get_name() + ", "

        cart_name.strip(', ')
        return cart_name

class CartItem(models.Model):
    """A cart item belongs to a cart. It stores the product and the amount of
    the product which has been taken into the cart.

    Instance variables:

    - product
       A reference to a product which is supposed to be bought
    - amount
       Amount of the product which is supposed to be bought.
    """
    cart = models.ForeignKey(Cart, verbose_name=_(u"Cart"))
    product = models.ForeignKey(Product, verbose_name=_(u"Product"))
    amount = models.IntegerField(_(u"Quantity"), blank=True, null=True)
    creation_date = models.DateTimeField(_(u"Creation date"), auto_now_add=True)
    modification_date = models.DateTimeField(_(u"Modification date"), auto_now=True, auto_now_add=True)

    class Meta:
        ordering = ['id']

    def get_price(self):
        """
        """
        return self.get_price_gross()

    def get_price_net(self):
        """Returns the total price of the cart item, which is just the
        multiplication of the product's price and the amount of the product
        within in the cart.
        """
        return self.product.get_price_net() * self.amount

    def get_price_gross(self):
        """Returns the gross price of the product.
        """
        return self.product.get_price_gross() * self.amount

    def get_tax(self):
        """Returns the absolute tax of the product.
        """
        return self.product.get_tax() * self.amount


# django imports
from django.core.exceptions import ObjectDoesNotExist
from django.contrib.auth.decorators import permission_required
from django.core.urlresolvers import reverse
from django.db import IntegrityError
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.template import RequestContext
from django.template.loader import render_to_string

# lfs imports
from lfs.catalog.models import Product
from lfs.catalog.models import ProductPropertyValue
from lfs.catalog.models import Property
from lfs.catalog.models import PropertyGroup
from lfs.core.signals import product_removed_property_group

@permission_required("manage_shop", login_url="/login/")
def manage_properties(request, product_id, template_name="manage/product/properties.html"):
    """
    """
    product = get_object_or_404(Product, pk=product_id)
    
    # Generate list of product property groups; used for enter value
    product_property_groups = []
    for property_group in product.property_groups.all():
        properties = []
        for property in property_group.properties.order_by("groupspropertiesrelation"):
            # Try to get the value, if it already exists.
            try:
                ppv = ProductPropertyValue.objects.get(property = property, product=product)
            except ProductPropertyValue.DoesNotExist:
                value = ""
            else: 
                value = ppv.value
            
            # mark selected options "selected"
            options = []
            for option in property.options.all():                
                options.append({
                    "id"       : option.id,
                    "name"     : option.name,
                    "selected" : str(option.id) == value
                })
            
            properties.append({
                "id"      : property.id,
                "name"    : property.name,
                "type"    : property.type,
                "options" : options,
                "value"   : value,
            })

        product_property_groups.append({
            "id"   : property_group.id,
            "name" : property_group.name,
            "properties" : properties,
        })
    
    # Generate list of all property groups; used for group selection
    product_property_group_ids = [p["id"] for p in product_property_groups]
    shop_property_groups = []
    for property_group in PropertyGroup.objects.all():
        
        shop_property_groups.append({
            "id" : property_group.id,
            "name" : property_group.name,
            "selected" : property_group.id in product_property_group_ids,
        })
    
    return render_to_string(template_name, RequestContext(request, {
        "product" : product,
        "product_property_groups" : product_property_groups,
        "shop_property_groups" : shop_property_groups,
    }))

@permission_required("manage_shop", login_url="/login/")
def update_property_groups(request, product_id):
    """Updates property groups for the product with passed id.
    """
    selected_group_ids = request.POST.getlist("selected-property-groups")
    
    for property_group in PropertyGroup.objects.all():
        # if the group is within selected groups we try to add it to the product
        # otherwise we try do delete it
        if str(property_group.id) in selected_group_ids:
            try:
                property_group.products.get(pk=product_id)
            except ObjectDoesNotExist:
                property_group.products.add(product_id)
        else:
            property_group.products.remove(product_id)
            product = Product.objects.get(pk=product_id)
            product_removed_property_group.send([property_group, product])
    
    url = reverse("lfs_manage_product", kwargs={"product_id" : product_id})        
    return HttpResponseRedirect(url)

@permission_required("manage_shop", login_url="/login/")    
def update_properties(request, product_id):
    """Updates properties for product with passed id.
    """
    # Update properties' values
    for key, value in request.POST.items():
        if key.startswith("property") == False:
            continue        

        property_id = key.split("-")[1]
        property = get_object_or_404(Property, pk=property_id)
        product = get_object_or_404(Product, pk=product_id)
        
        try:
            ppv = ProductPropertyValue.objects.get(product = product_id, property = property_id)
        except ProductPropertyValue.DoesNotExist:
            if not property.is_valid_value(value):
                value = 0
            ProductPropertyValue.objects.create(product=product, property = property, value=value)
        else:            
            if not property.is_valid_value(value):
                value = 0
            
            ppv.value = value
            ppv.save()

    url = reverse("lfs_manage_product", kwargs={"product_id" : product_id})    
    return HttpResponseRedirect(url)




from south.db import db
from django.db import models
from lfs.tax.models import *

class Migration:
    
    def forwards(self, orm):
        
        # Adding model 'Tax'
        db.create_table('tax_tax', (
            ('rate', models.FloatField(_(u"Rate"), default=0)),
            ('id', models.AutoField(primary_key=True)),
            ('description', models.TextField(_(u"Description"), blank=True)),
        ))
        db.send_create_signal('tax', ['Tax'])
        
    
    
    def backwards(self, orm):
        
        # Deleting model 'Tax'
        db.delete_table('tax_tax')
        
    
    
    models = {
        'tax.tax': {
            'description': ('models.TextField', ['_(u"Description")'], {'blank': 'True'}),
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'rate': ('models.FloatField', ['_(u"Rate")'], {'default': '0'})
        }
    }
    
    complete_apps = ['tax']




from south.db import db
from django.db import models
from lfs.marketing.models import *

class Migration:
    
    def forwards(self, orm):
        
        # Adding model 'Topseller'
        db.create_table('marketing_topseller', (
            ('position', models.PositiveSmallIntegerField(_(u"Position"), default=1)),
            ('product', models.ForeignKey(orm['catalog.Product'], verbose_name=_(u"Product"))),
            ('id', models.AutoField(primary_key=True)),
        ))
        db.send_create_signal('marketing', ['Topseller'])
        
    
    
    def backwards(self, orm):
        
        # Deleting model 'Topseller'
        db.delete_table('marketing_topseller')
        
    
    
    models = {
        'marketing.topseller': {
            'Meta': {'ordering': '["position"]'},
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'position': ('models.PositiveSmallIntegerField', ['_(u"Position")'], {'default': '1'}),
            'product': ('models.ForeignKey', ['Product'], {'verbose_name': '_(u"Product")'})
        },
        'catalog.product': {
            'Meta': {'ordering': '("name",)'},
            '_stub': True,
            'id': ('models.AutoField', [], {'primary_key': 'True'})
        }
    }
    
    complete_apps = ['marketing']




from south.db import db
from django.db import models
from lfs.shipping.models import *

class Migration:
    
    def forwards(self, orm):
        
        # Adding model 'ShippingMethodPrice'
        db.create_table('shipping_shippingmethodprice', (
            ('id', models.AutoField(primary_key=True)),
            ('shipping_method', models.ForeignKey(orm.ShippingMethod, related_name="prices", verbose_name=_(u"shipping_method"))),
            ('price', models.FloatField(_(u"Price"), default=0.0)),
            ('priority', models.IntegerField(_(u"Priority"), default=0)),
            ('active', models.BooleanField(_(u"Active"), default=True)),
        ))
        db.send_create_signal('shipping', ['ShippingMethodPrice'])
        
        # Adding model 'ShippingMethod'
        db.create_table('shipping_shippingmethod', (
            ('id', models.AutoField(primary_key=True)),
            ('name', models.CharField(_(u"Name"), max_length=50)),
            ('description', models.TextField(_(u"Description"), blank=True)),
            ('note', models.TextField(_(u"Note"), blank=True)),
            ('priority', models.IntegerField(_(u"Priority"), default=0)),
            ('image', models.ImageField(_(u"Image"), null=True, blank=True)),
            ('active', models.BooleanField(_(u"Active"), default=False)),
            ('tax', models.ForeignKey(orm['tax.Tax'], null=True, verbose_name=_(u"Tax"), blank=True)),
            ('price', models.FloatField(_(u"Price"), default=0.0)),
            ('delivery_time', models.ForeignKey(orm['catalog.DeliveryTime'], null=True, verbose_name=_(u"Delivery time"), blank=True)),
        ))
        db.send_create_signal('shipping', ['ShippingMethod'])
        
    
    
    def backwards(self, orm):
        
        # Deleting model 'ShippingMethodPrice'
        db.delete_table('shipping_shippingmethodprice')
        
        # Deleting model 'ShippingMethod'
        db.delete_table('shipping_shippingmethod')
        
    
    
    models = {
        'shipping.shippingmethodprice': {
            'Meta': {'ordering': '("priority",)'},
            'active': ('models.BooleanField', ['_(u"Active")'], {'default': 'True'}),
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'price': ('models.FloatField', ['_(u"Price")'], {'default': '0.0'}),
            'priority': ('models.IntegerField', ['_(u"Priority")'], {'default': '0'}),
            'shipping_method': ('models.ForeignKey', ["orm['shipping.ShippingMethod']"], {'related_name': '"prices"', 'verbose_name': '_(u"shipping_method")'})
        },
        'catalog.deliverytime': {
            'Meta': {'ordering': '("min",)'},
            '_stub': True,
            'id': ('models.AutoField', [], {'primary_key': 'True'})
        },
        'tax.tax': {
            '_stub': True,
            'id': ('models.AutoField', [], {'primary_key': 'True'})
        },
        'shipping.shippingmethod': {
            'Meta': {'ordering': '("priority",)'},
            'active': ('models.BooleanField', ['_(u"Active")'], {'default': 'False'}),
            'delivery_time': ('models.ForeignKey', ["orm['catalog.DeliveryTime']"], {'null': 'True', 'verbose_name': '_(u"Delivery time")', 'blank': 'True'}),
            'description': ('models.TextField', ['_(u"Description")'], {'blank': 'True'}),
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'image': ('models.ImageField', ['_(u"Image")'], {'null': 'True', 'blank': 'True'}),
            'name': ('models.CharField', ['_(u"Name")'], {'max_length': '50'}),
            'note': ('models.TextField', ['_(u"Note")'], {'blank': 'True'}),
            'price': ('models.FloatField', ['_(u"Price")'], {'default': '0.0'}),
            'priority': ('models.IntegerField', ['_(u"Priority")'], {'default': '0'}),
            'tax': ('models.ForeignKey', ["orm['tax.Tax']"], {'null': 'True', 'verbose_name': '_(u"Tax")', 'blank': 'True'})
        }
    }
    
    complete_apps = ['shipping']




from south.db import db
from django.db import models
from lfs.criteria.models import *

class Migration:
    
    def forwards(self, orm):
        
        # Adding model 'WeightCriterion'
        db.create_table('criteria_weightcriterion', (
            ('operator', models.PositiveIntegerField(_(u"Operator"), null=True, blank=True)),
            ('id', models.AutoField(primary_key=True)),
            ('weight', models.FloatField(_(u"Weight"), default=0.0)),
        ))
        db.send_create_signal('criteria', ['WeightCriterion'])
        
        # Adding model 'UserCriterion'
        db.create_table('criteria_usercriterion', (
            ('id', models.AutoField(primary_key=True)),
        ))
        db.send_create_signal('criteria', ['UserCriterion'])
        
        # Adding model 'HeightCriterion'
        db.create_table('criteria_heightcriterion', (
            ('operator', models.PositiveIntegerField(null=True, blank=True)),
            ('id', models.AutoField(primary_key=True)),
            ('height', models.FloatField(_(u"Height"), default=0.0)),
        ))
        db.send_create_signal('criteria', ['HeightCriterion'])
        
        # Adding model 'CriteriaObjects'
        db.create_table('criteria_criteriaobjects', (
            ('criterion_id', models.PositiveIntegerField(_(u"Content id"))),
            ('content_type', models.ForeignKey(orm['contenttypes.ContentType'], related_name="content_type", verbose_name=_(u"Content type"))),
            ('position', models.PositiveIntegerField(_(u"Position"), default=999)),
            ('content_id', models.PositiveIntegerField(_(u"Content id"))),
            ('id', models.AutoField(primary_key=True)),
            ('criterion_type', models.ForeignKey(orm['contenttypes.ContentType'], related_name="criterion", verbose_name=_(u"Criterion type"))),
        ))
        db.send_create_signal('criteria', ['CriteriaObjects'])
        
        # Adding model 'CountryCriterion'
        db.create_table('criteria_countrycriterion', (
            ('operator', models.PositiveIntegerField(_(u"Operator"), null=True, blank=True)),
            ('id', models.AutoField(primary_key=True)),
        ))
        db.send_create_signal('criteria', ['CountryCriterion'])
        
        # Adding model 'LengthCriterion'
        db.create_table('criteria_lengthcriterion', (
            ('operator', models.PositiveIntegerField(_(u"Operator"), null=True, blank=True)),
            ('length', models.FloatField(_(u"Length"), default=0.0)),
            ('id', models.AutoField(primary_key=True)),
        ))
        db.send_create_signal('criteria', ['LengthCriterion'])
        
        # Adding model 'CartPriceCriterion'
        db.create_table('criteria_cartpricecriterion', (
            ('operator', models.PositiveIntegerField(_(u"Operator"), null=True, blank=True)),
            ('price', models.FloatField(_(u"Price"), default=0.0)),
            ('id', models.AutoField(primary_key=True)),
        ))
        db.send_create_signal('criteria', ['CartPriceCriterion'])
        
        # Adding model 'WidthCriterion'
        db.create_table('criteria_widthcriterion', (
            ('operator', models.PositiveIntegerField(_(u"Operator"), null=True, blank=True)),
            ('width', models.FloatField(_(u"Width"), default=0.0)),
            ('id', models.AutoField(primary_key=True)),
        ))
        db.send_create_signal('criteria', ['WidthCriterion'])
        
        # Adding ManyToManyField 'UserCriterion.users'
        db.create_table('criteria_usercriterion_users', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('usercriterion', models.ForeignKey(UserCriterion, null=False)),
            ('user', models.ForeignKey(User, null=False))
        ))
        
        # Adding ManyToManyField 'CountryCriterion.countries'
        db.create_table('criteria_countrycriterion_countries', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('countrycriterion', models.ForeignKey(CountryCriterion, null=False)),
            ('country', models.ForeignKey(Country, null=False))
        ))
        
    
    
    def backwards(self, orm):
        
        # Deleting model 'WeightCriterion'
        db.delete_table('criteria_weightcriterion')
        
        # Deleting model 'UserCriterion'
        db.delete_table('criteria_usercriterion')
        
        # Deleting model 'HeightCriterion'
        db.delete_table('criteria_heightcriterion')
        
        # Deleting model 'CriteriaObjects'
        db.delete_table('criteria_criteriaobjects')
        
        # Deleting model 'CountryCriterion'
        db.delete_table('criteria_countrycriterion')
        
        # Deleting model 'LengthCriterion'
        db.delete_table('criteria_lengthcriterion')
        
        # Deleting model 'CartPriceCriterion'
        db.delete_table('criteria_cartpricecriterion')
        
        # Deleting model 'WidthCriterion'
        db.delete_table('criteria_widthcriterion')
        
        # Dropping ManyToManyField 'UserCriterion.users'
        db.delete_table('criteria_usercriterion_users')
        
        # Dropping ManyToManyField 'CountryCriterion.countries'
        db.delete_table('criteria_countrycriterion_countries')
        
    
    
    models = {
        'criteria.weightcriterion': {
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'operator': ('models.PositiveIntegerField', ['_(u"Operator")'], {'null': 'True', 'blank': 'True'}),
            'weight': ('models.FloatField', ['_(u"Weight")'], {'default': '0.0'})
        },
        'criteria.heightcriterion': {
            'height': ('models.FloatField', ['_(u"Height")'], {'default': '0.0'}),
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'operator': ('models.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'})
        },
        'auth.user': {
            '_stub': True,
            'id': ('models.AutoField', [], {'primary_key': 'True'})
        },
        'criteria.criteriaobjects': {
            'Meta': {'ordering': '["position"]'},
            'content_id': ('models.PositiveIntegerField', ['_(u"Content id")'], {}),
            'content_type': ('models.ForeignKey', ['ContentType'], {'related_name': '"content_type"', 'verbose_name': '_(u"Content type")'}),
            'criterion_id': ('models.PositiveIntegerField', ['_(u"Content id")'], {}),
            'criterion_type': ('models.ForeignKey', ['ContentType'], {'related_name': '"criterion"', 'verbose_name': '_(u"Criterion type")'}),
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'position': ('models.PositiveIntegerField', ['_(u"Position")'], {'default': '999'})
        },
        'criteria.usercriterion': {
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'users': ('models.ManyToManyField', ['User'], {})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label','model'),)", 'db_table': "'django_content_type'"},
            '_stub': True,
            'id': ('models.AutoField', [], {'primary_key': 'True'})
        },
        'criteria.lengthcriterion': {
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'length': ('models.FloatField', ['_(u"Length")'], {'default': '0.0'}),
            'operator': ('models.PositiveIntegerField', ['_(u"Operator")'], {'null': 'True', 'blank': 'True'})
        },
        'core.country': {
            'Meta': {'ordering': '("name",)'},
            '_stub': True,
            'id': ('models.AutoField', [], {'primary_key': 'True'})
        },
        'criteria.countrycriterion': {
            'countries': ('models.ManyToManyField', ['Country'], {'verbose_name': '_(u"Countries")'}),
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'operator': ('models.PositiveIntegerField', ['_(u"Operator")'], {'null': 'True', 'blank': 'True'})
        },
        'criteria.cartpricecriterion': {
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'operator': ('models.PositiveIntegerField', ['_(u"Operator")'], {'null': 'True', 'blank': 'True'}),
            'price': ('models.FloatField', ['_(u"Price")'], {'default': '0.0'})
        },
        'criteria.widthcriterion': {
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'operator': ('models.PositiveIntegerField', ['_(u"Operator")'], {'null': 'True', 'blank': 'True'}),
            'width': ('models.FloatField', ['_(u"Width")'], {'default': '0.0'})
        }
    }
    
    complete_apps = ['criteria']



# django imports
from django import forms
from django.db import models
from django.template.loader import render_to_string

# portlets
from portlets.models import Portlet

# lfs imports
import lfs.core.utils

class CategoriesPortlet(Portlet):
    """A portlet to display categories.
    """
    start_level = models.PositiveSmallIntegerField(default=1)
    expand_level = models.PositiveSmallIntegerField(default=1)

    class Meta:
        app_label = 'portlet'

    def __unicode__(self):
        return "%s" % self.id

    def render(self, context):
        """Renders the portlet as html.
        """
        # Calculate current categories
        request = context.get("request")

        object = context.get("category") or context.get("product")
        current_categories = lfs.core.utils.get_current_categories(request, object)

        ct = lfs.core.utils.CategoryTree(
            current_categories, self.start_level, self.expand_level)
        category_tree = ct.get_category_tree()

        return render_to_string("lfs/portlets/categories.html", {
            "title" : self.title,
            "categories" : category_tree,
            "MEDIA_URL" : context.get("MEDIA_URL"),
        })

    def form(self, **kwargs):
        """
        """
        return CategoriesPortletForm(instance=self, **kwargs)

class CategoriesPortletForm(forms.ModelForm):
    """
    """
    class Meta:
        model = CategoriesPortlet



# django imports
from django.conf import settings
from django.contrib.sites.models import Site
from django.core.urlresolvers import reverse

# lfs imports
import lfs.core.utils
from lfs.caching.utils import lfs_get_object_or_404
from lfs.core.models import Shop
from lfs.criteria import utils as criteria_utils
from lfs.customer import utils as customer_utils
from lfs.payment.models import PaymentMethod
from lfs.payment.settings import CREDIT_CARD
from lfs.payment.settings import PAYPAL

# other imports
from paypal.standard.conf import POSTBACK_ENDPOINT, SANDBOX_POSTBACK_ENDPOINT

def update_to_valid_payment_method(request, customer, save=False):
    """After this method has been called the given customer has a valid
    payment method.
    """
    valid_sms = get_valid_payment_methods(request)

    if customer.selected_payment_method not in valid_sms:
        customer.selected_payment_method = get_default_payment_method(request)
        if save:
            customer.save()

def get_valid_payment_methods(request):
    """Returns all valid payment methods (aka. selectable) for given request as
    list.
    """
    result = []
    for pm in PaymentMethod.objects.filter(active=True):
        if criteria_utils.is_valid(request, pm):
            result.append(pm)
    return result

def get_default_payment_method(request):
    """Returns the default payment method for given request.
    """
    active_payment_methods = PaymentMethod.objects.filter(active=True)
    return criteria_utils.get_first_valid(request, active_payment_methods)

def get_selected_payment_method(request):
    """Returns the selected payment method for given request. This could either
    be an explicitly selected payment method of the current user or the default
    payment method.
    """
    customer = customer_utils.get_customer(request)
    if customer and customer.selected_payment_method:
        return customer.selected_payment_method
    else:
        return get_default_payment_method(request)

def get_payment_costs(request, payment_method):
    """Returns the payment price and tax for the given request.
    """
    if payment_method is None:
        return {
            "price" : 0.0,
            "tax" : 0.0
        }

    try:
        tax_rate = payment_method.tax.rate
    except AttributeError:
        tax_rate = 0.0

    price = criteria_utils.get_first_valid(request,
        payment_method.prices.all())

    if price is None:
        price = payment_method.price
        tax = (tax_rate/(tax_rate+100)) * price

        return {
            "price" : price,
            "tax" : tax
        }
    else:
        tax = (tax_rate/(tax_rate+100)) * price.price

        return {
            "price" : price.price,
            "tax" : tax
        }

def process_payment(request):
    """Processes the payment depending on the selected payment method. Returns
    a dictionary with the success state, the next url and a optional error
    message.
    """
    payment_method = get_selected_payment_method(request)
    shop = lfs.core.utils.get_default_shop()

    if payment_method.id == PAYPAL and settings.LFS_PAYPAL_REDIRECT:
        return {
            "success" : True,
            "next-url" : "this is set within checkout.views",
        }
    elif payment_method.id == CREDIT_CARD:
        module = lfs.core.utils.import_module(settings.LFS_CREDIT_CARD_MODULE)
        return module.process(request)
    return {
        "success" : True,
        "next-url" : reverse("lfs_thank_you"),
    }

def create_paypal_link_for_order(order):
    """Creates paypal link for given order.
    """
    shop = lfs_get_object_or_404(Shop, pk=1)
    current_site = Site.objects.get(id=settings.SITE_ID)

    info = {
        "cmd" : "_xclick",
        "upload" : "1",
        "business" : settings.PAYPAL_RECEIVER_EMAIL,
        "currency_code" : shop.default_currency,
        "notify_url" : "http://" + current_site.domain + reverse('paypal-ipn'),
        "return" : "http://" + current_site.domain + reverse('paypal-pdt'),
        "first_name" : order.invoice_firstname,
        "last_name" : order.invoice_lastname,
        "address1" : order.invoice_street,
        "address2" : "",
        "city" : order.invoice_city,
        "state" : order.invoice_state,
        "zip" : order.invoice_zip_code,
        "no_shipping" : "1",
        "custom": order.uuid,
        "invoice": order.uuid,
        "item_name" : shop.shop_owner,
        "amount" : order.price - float("%.2f" % order.tax),
        "tax" : "%.2f" % order.tax,
    }
    
    parameters = "&".join(["%s=%s" % (k, v) for (k, v) in info.items()])
    if settings.DEBUG:
        url = SANDBOX_POSTBACK_ENDPOINT + "?" + parameters
    else:
        url = POSTBACK_ENDPOINT + "?" + parameters

    return url
    
def create_paypal_link_for_request(request):
    """Creates paypal link for given request.
    """
    shop = lfs_get_object_or_404(Shop, pk=1)
    current_site = Site.objects.get(id=settings.SITE_ID)

    customer = lfs.customer.utils.get_customer(request)
    invoice_address = customer.selected_invoice_address

    cart = lfs.cart.utils.get_cart(request)
    cart_price, cart_tax = lfs.cart.utils.get_cart_costs()

    info = {
        "cmd" : "_xclick",
        "upload" : "1",
        "business" : settings.PAYPAL_RECEIVER_EMAIL,
        "currency_code" : shop.default_currency,
        "notify_url" : "http://" + current_site.domain + reverse('paypal-ipn'),
        "return" : "http://" + current_site.domain + reverse('paypal-pdt'),
        "first_name" : invoice_address.firstname,
        "last_name" : invoice_address.lastname,
        "address1" : invoice_address.street,
        "address2" : "",
        "city" : invoice_address.city,
        "state" : invoice_address.state,
        "zip" : invoice_address.zip_code,
        "no_shipping" : "1",
        "custom": cart.id,
        "invoice": cart.id,
        "item_name" : shop.shop_owner,
        "amount" : "%.2f" % (cart_price - cart_tax),
        "tax" : "%.2f" % cart_tax,
    }

    parameters = "&".join(["%s=%s" % (k, v) for (k, v) in info.items()])
    if settings.DEBUG:
        url = SANDBOX_POSTBACK_ENDPOINT + "?" + parameters
    else:
        url = POSTBACK_ENDPOINT + "?" + parameters

    return url


# python imports
from datetime import datetime
from datetime import timedelta

# django imports
from django.core.cache import cache
from django.db.models import Q

# lfs imports
from lfs.catalog.models import Product
from lfs.marketing.models import Topseller
from lfs.order.models import Order
from lfs.order.settings import CLOSED
from lfs.order.models import OrderItem
from django.db import connection

def get_orders(days=14):
    """Returns closed orders which are closed for given amount of days.
    """
    limit = datetime.now() - timedelta(days=days)

    orders = Order.objects.filter(state=CLOSED, state_modified__lte=limit)
    return orders

def get_topseller(limit=5):
    """Returns products with the most sales. Limited by given limit.
    """
    cache_key = "topseller"
    topseller = cache.get(cache_key)
    if topseller is not None:
        return topseller

    # TODO: Check Django 1.1's aggregation
    cursor = connection.cursor()
    cursor.execute("""SELECT product_id, sum(product_amount) as sum
                      FROM order_orderitem
                      GROUP BY product_id
                      ORDER BY sum DESC limit %s""" % (limit*2))

    products = []
    for topseller in cursor.fetchall():
        product = Product.objects.get(pk=topseller[0])
        if product.is_active():
            try:
                products.append(product)
            except Product.DoesNotExist:
                pass

    for explicit_ts in Topseller.objects.all():

        if explicit_ts.product.is_active():
            # Remove explicit_ts if it's already in the object list
            if explicit_ts.product in products:
                products.pop(products.index(explicit_ts.product))

            # Then reinsert the explicit_ts on the given position
            position = explicit_ts.position - 1
            if position < 0:
                position = 0
            products.insert(position, explicit_ts.product)

    products = products[:limit]
    cache.set(cache_key, products)
    return products

def get_topseller_for_category(category, limit=5):
    """Returns products with the most sales withing given category. Limited by
    given limit.
    """
    # TODO: Check Django 1.1's aggregation

    cache_key = "topseller-%s" % category.id
    topseller = cache.get(cache_key)
    if topseller is not None:
        return topseller

    # 1. Get all sub catgegories of passed category
    categories = [category]
    categories.extend(category.get_all_children())
    category_ids = [c.id for c in categories]
    
    # 2. Get all order items with products within these categories.
    #    product__parent__categories is for variants
    f = Q(product__parent__categories__in=category_ids) | \
        Q(product__categories__in=category_ids)
        
    order_items = OrderItem.objects.filter(f)
    
    # 3. Calculate totals per product
    products = {}
    for order_item in order_items:
        if order_item.product.is_active():
            if not products.has_key(order_item.product.id):
                products[order_item.product.id] = 0
            products[order_item.product.id] += order_item.product_amount

    # 4. Sort product ids on values
    products = products.items()
    products.sort(lambda a, b: cmp(b[1], a[1]))

    objects = []
    for product_id, quantity in products[:limit]:
        try:
            objects.append(Product.objects.get(pk=product_id))
        except Product.DoesNotExist:
            pass

    for explicit_ts in Topseller.objects.filter(product__categories__in=category_ids):
        
        if explicit_ts.product.is_active():
            # Remove explicit_ts if it's already in the object list
            if explicit_ts.product in objects:
                objects.pop(objects.index(explicit_ts.product))

            # Then reinsert the explicit_ts on the given position
            position = explicit_ts.position - 1
            if position < 0:
                position = 0
            objects.insert(position, explicit_ts.product)

    objects = objects[:limit]
    cache.set(cache_key, objects)
    return objects



from south.db import db
from django.db import models
from lfs.cart.models import *

class Migration:
    
    def forwards(self, orm):
        
        # Adding field 'CartItem.creation_date'
        db.add_column('cart_cartitem', 'creation_date', models.DateTimeField(_(u"Creation date"), auto_now_add=True))
        
        # Adding field 'Cart.creation_date'
        db.add_column('cart_cart', 'creation_date', models.DateTimeField(_(u"Creation date"), auto_now_add=True))
        
        # Adding field 'CartItem.modification_date'
        db.add_column('cart_cartitem', 'modification_date', models.DateTimeField(_(u"Modification date"), auto_now=True, auto_now_add=True))
        
        # Adding field 'Cart.modification_date'
        db.add_column('cart_cart', 'modification_date', models.DateTimeField(_(u"Modification date"), auto_now=True, auto_now_add=True))
        
    def backwards(self, orm):
        
        # Deleting field 'CartItem.creation_date'
        db.delete_column('cart_cartitem', 'creation_date')
        
        # Deleting field 'Cart.creation_date'
        db.delete_column('cart_cart', 'creation_date')
        
        # Deleting field 'CartItem.modification_date'
        db.delete_column('cart_cartitem', 'modification_date')
        
        # Deleting field 'Cart.modification_date'
        db.delete_column('cart_cart', 'modification_date')
            
    models = {
        'auth.user': {
            '_stub': True,
            'id': ('models.AutoField', [], {'primary_key': 'True'})
        },
        'catalog.product': {
            'Meta': {'ordering': '("name",)'},
            '_stub': True,
            'id': ('models.AutoField', [], {'primary_key': 'True'})
        },
        'cart.cartitem': {
            'Meta': {'ordering': "['id']"},
            'amount': ('models.IntegerField', ['_(u"Quantity")'], {'null': 'True', 'blank': 'True'}),
            'cart': ('models.ForeignKey', ["orm['cart.Cart']"], {}),
            'creation_date': ('models.DateTimeField', ['_(u"Creation date")'], {'auto_now_add': 'True'}),
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'modification_date': ('models.DateTimeField', ['_(u"Modification date")'], {'auto_now': 'True', 'auto_now_add': 'True'}),
            'product': ('models.ForeignKey', ["orm['catalog.Product']"], {})
        },
        'cart.cart': {
            'creation_date': ('models.DateTimeField', ['_(u"Creation date")'], {'auto_now_add': 'True'}),
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'modification_date': ('models.DateTimeField', ['_(u"Modification date")'], {'auto_now': 'True', 'auto_now_add': 'True'}),
            'session': ('models.CharField', ['_(u"Session")'], {'max_length': '100', 'blank': 'True'}),
            'user': ('models.ForeignKey', ["orm['auth.User']"], {'null': 'True', 'blank': 'True'})
        }
    }
    
    complete_apps = ['cart']




from south.db import db
from django.db import models
from lfs.customer.models import *

class Migration:
    
    def forwards(self, orm):
        
        # Adding field 'Address.company_name'
        db.add_column('customer_address', 'company_name', models.CharField(_("Company name"), max_length=50))
        
    
    
    def backwards(self, orm):
        
        # Deleting field 'Address.company_name'
        db.delete_column('customer_address', 'company_name')
        
    
    
    models = {
        'customer.creditcard': {
            'customer': ('models.ForeignKey', ["orm['customer.Customer']"], {'related_name': '"credit_cards"', 'null': 'True', 'blank': 'True'}),
            'expiration_date_month': ('models.IntegerField', ['_(u"Expiration date month")'], {}),
            'expiration_date_year': ('models.IntegerField', ['_(u"Expiration date year")'], {}),
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'number': ('models.CharField', ['_(u"Number")'], {'max_length': '30', 'blank': 'True'}),
            'owner': ('models.CharField', ['_(u"Owner")'], {'max_length': '100', 'blank': 'True'}),
            'type': ('models.CharField', ['_(u"Type")'], {'max_length': '30', 'blank': 'True'})
        },
        'payment.paymentmethod': {
            'Meta': {'ordering': '("priority",)'},
            '_stub': True,
            'id': ('models.AutoField', [], {'primary_key': 'True'})
        },
        'customer.customer': {
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'selected_bank_account': ('models.ForeignKey', ["orm['customer.BankAccount']"], {'related_name': '"selected_bank_account"', 'null': 'True', 'blank': 'True'}),
            'selected_country': ('models.ForeignKey', ["orm['core.Country']"], {'null': 'True', 'blank': 'True'}),
            'selected_credit_card': ('models.ForeignKey', ["orm['customer.CreditCard']"], {'related_name': '"selected_credit_card"', 'null': 'True', 'blank': 'True'}),
            'selected_invoice_address': ('models.ForeignKey', ["orm['customer.Address']"], {'related_name': '"selected_invoice_address"', 'null': 'True', 'blank': 'True'}),
            'selected_payment_method': ('models.ForeignKey', ["orm['payment.PaymentMethod']"], {'related_name': '"selected_payment_method"', 'null': 'True', 'blank': 'True'}),
            'selected_shipping_address': ('models.ForeignKey', ["orm['customer.Address']"], {'related_name': '"selected_shipping_address"', 'null': 'True', 'blank': 'True'}),
            'selected_shipping_method': ('models.ForeignKey', ["orm['shipping.ShippingMethod']"], {'related_name': '"selected_shipping_method"', 'null': 'True', 'blank': 'True'}),
            'session': ('models.CharField', [], {'max_length': '100', 'blank': 'True'}),
            'user': ('models.ForeignKey', ["orm['auth.User']"], {'null': 'True', 'blank': 'True'})
        },
        'auth.user': {
            '_stub': True,
            'id': ('models.AutoField', [], {'primary_key': 'True'})
        },
        'shipping.shippingmethod': {
            'Meta': {'ordering': '("priority",)'},
            '_stub': True,
            'id': ('models.AutoField', [], {'primary_key': 'True'})
        },
        'customer.address': {
            'city': ('models.CharField', ['_("City")'], {'max_length': '50'}),
            'company_name': ('models.CharField', ['_("Company name")'], {'max_length': '50'}),
            'country': ('models.ForeignKey', ["orm['core.Country']"], {}),
            'customer': ('models.ForeignKey', ["orm['customer.Customer']"], {'related_name': '"addresses"', 'null': 'True', 'blank': 'True'}),
            'email': ('models.EmailField', ['_("E-Mail")'], {'max_length': '50', 'null': 'True', 'blank': 'True'}),
            'firstname': ('models.CharField', ['_("Firstname")'], {'max_length': '50'}),
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'lastname': ('models.CharField', ['_("Lastname")'], {'max_length': '50'}),
            'phone': ('models.CharField', ['_("Phone")'], {'max_length': '20', 'blank': 'True'}),
            'state': ('models.CharField', ['_("State")'], {'max_length': '50', 'blank': 'True'}),
            'street': ('models.CharField', ['_("Street")'], {'max_length': '100'}),
            'zip_code': ('models.CharField', ['_("Zip code")'], {'max_length': '10'})
        },
        'customer.bankaccount': {
            'account_number': ('models.CharField', ['_(u"Account number")'], {'max_length': '30', 'blank': 'True'}),
            'bank_identification_code': ('models.CharField', ['_(u"Bank identification code")'], {'max_length': '30', 'blank': 'True'}),
            'bank_name': ('models.CharField', ['_(u"Bank name")'], {'max_length': '100', 'blank': 'True'}),
            'customer': ('models.ForeignKey', ["orm['customer.Customer']"], {'related_name': '"bank_accounts"', 'null': 'True', 'blank': 'True'}),
            'depositor': ('models.CharField', ['_(u"Depositor")'], {'max_length': '100', 'blank': 'True'}),
            'id': ('models.AutoField', [], {'primary_key': 'True'})
        },
        'core.country': {
            'Meta': {'ordering': '("name",)'},
            '_stub': True,
            'id': ('models.AutoField', [], {'primary_key': 'True'})
        }
    }
    
    complete_apps = ['customer']




from south.db import db
from django.db import models
from lfs.catalog.models import *

class Migration:
    
    def forwards(self, orm):
        
        # Adding model 'ProductPropertyValue'
        db.create_table('catalog_productpropertyvalue', (
            ('id', models.AutoField(primary_key=True)),
            ('product', models.ForeignKey(orm.Product, related_name="property_values", verbose_name=_(u"Product"))),
            ('parent_id', models.IntegerField(null=True, blank=True)),
            ('property', models.ForeignKey(orm.Property, related_name="property_values", verbose_name=_(u"Property"))),
            ('value', models.CharField(max_length=100, blank=True)),
            ('value_as_float', models.FloatField(null=True, blank=True)),
        ))
        db.send_create_signal('catalog', ['ProductPropertyValue'])
        
        # Adding model 'StaticBlock'
        db.create_table('catalog_staticblock', (
            ('id', models.AutoField(primary_key=True)),
            ('name', models.CharField(_(u"Name"), max_length=30)),
            ('html', models.TextField(_(u"HTML"), blank=True)),
        ))
        db.send_create_signal('catalog', ['StaticBlock'])
        
        # Adding model 'PropertyGroup'
        db.create_table('catalog_propertygroup', (
            ('id', models.AutoField(primary_key=True)),
            ('name', models.CharField(max_length=50, blank=True)),
        ))
        db.send_create_signal('catalog', ['PropertyGroup'])
        
        # Adding model 'Category'
        db.create_table('catalog_category', (
            ('id', models.AutoField(primary_key=True)),
            ('name', models.CharField(_(u"Name"), max_length=50)),
            ('slug', models.SlugField(_(u"Slug"), unique=True)),
            ('parent', models.ForeignKey(orm.Category, null=True, verbose_name=_(u"Parent"), blank=True)),
            ('show_all_products', models.BooleanField(_(u"Show all products"), default=True)),
            ('short_description', models.TextField(_(u"Short description"), blank=True)),
            ('description', models.TextField(_(u"Description"), blank=True)),
            ('image', ImageWithThumbsField(_(u"Image"), null=True, sizes=((60,60),(100,100),(200,200),(400,400)), blank=True)),
            ('position', models.IntegerField(_(u"Position"), default=1000)),
            ('static_block', models.ForeignKey(orm.StaticBlock, related_name="categories", null=True, verbose_name=_(u"Static block"), blank=True)),
            ('content', models.IntegerField(_(u"Content"), default=1)),
            ('active_formats', models.BooleanField(_(u"Active formats"), default=False)),
            ('product_rows', models.IntegerField(_(u"Product rows"), default=3)),
            ('product_cols', models.IntegerField(_(u"Product cols"), default=3)),
            ('category_cols', models.IntegerField(_(u"Category cols"), default=3)),
            ('meta_keywords', models.TextField(_(u"Meta keywords"), blank=True)),
            ('meta_description', models.TextField(_(u"Meta description"), blank=True)),
        ))
        db.send_create_signal('catalog', ['Category'])
        
        # Adding model 'ProductAccessories'
        db.create_table('catalog_productaccessories', (
            ('id', models.AutoField(primary_key=True)),
            ('product', models.ForeignKey(orm.Product, related_name="productaccessories_product", verbose_name=_(u"Product"))),
            ('accessory', models.ForeignKey(orm.Product, related_name="productaccessories_accessory", verbose_name=_(u"Acessory"))),
            ('position', models.IntegerField(_(u"Position"), default=999)),
            ('quantity', models.FloatField(_(u"Quantity"), default=1)),
        ))
        db.send_create_signal('catalog', ['ProductAccessories'])
        
        # Adding model 'FilterStep'
        db.create_table('catalog_filterstep', (
            ('id', models.AutoField(primary_key=True)),
            ('property', models.ForeignKey(orm.Property, related_name="steps", verbose_name=_(u"Property"))),
            ('start', models.FloatField()),
        ))
        db.send_create_signal('catalog', ['FilterStep'])
        
        # Adding model 'Product'
        db.create_table('catalog_product', (
            ('id', models.AutoField(primary_key=True)),
            ('name', models.CharField(_(u"Name"), max_length=80)),
            ('slug', models.SlugField(_(u"Slug"), max_length=80, unique=True)),
            ('sku', models.CharField(_(u"SKU"), max_length=30, blank=True)),
            ('price', models.FloatField(_(u"Price"), default=0.0)),
            ('effective_price', models.FloatField(_(u"Price"), blank=True)),
            ('short_description', models.TextField(_(u"Short description"), blank=True)),
            ('description', models.TextField(_(u"Description"), blank=True)),
            ('meta_keywords', models.TextField(_(u"Meta keywords"), blank=True)),
            ('meta_description', models.TextField(_(u"Meta description"), blank=True)),
            ('for_sale', models.BooleanField(_(u"For sale"), default=False)),
            ('for_sale_price', models.FloatField(_(u"For sale price"), default=0.0)),
            ('active', models.BooleanField(_(u"Active"), default=False)),
            ('creation_date', models.DateTimeField(_(u"Creation date"), auto_now_add=True)),
            ('deliverable', models.BooleanField(_(u"Deliverable"), default=True)),
            ('manual_delivery_time', models.BooleanField(_(u"Manual delivery time"), default=False)),
            ('delivery_time', models.ForeignKey(orm.DeliveryTime, related_name="products_delivery_time", null=True, verbose_name=_(u"Delivery time"), blank=True)),
            ('order_time', models.ForeignKey(orm.DeliveryTime, related_name="products_order_time", null=True, verbose_name=_(u"Order time"), blank=True)),
            ('ordered_at', models.DateField(_(u"Ordered at"), null=True, blank=True)),
            ('manage_stock_amount', models.BooleanField(_(u"Manage stock amount"), default=True)),
            ('stock_amount', models.DecimalField(_(u"Stock amount"), default=0, max_digits=3, decimal_places=1)),
            ('weight', models.FloatField(_(u"Weight"), default=0.0)),
            ('height', models.FloatField(_(u"Height"), default=0.0)),
            ('length', models.FloatField(_(u"Length"), default=0.0)),
            ('width', models.FloatField(_(u"Width"), default=0.0)),
            ('tax', models.ForeignKey(orm['tax.Tax'], null=True, verbose_name=_(u"Tax"), blank=True)),
            ('sub_type', models.CharField(_(u"Subtype"), default='0', max_length=10)),
            ('default_variant', models.ForeignKey(orm.Product, null=True, verbose_name=_(u"Default variant"), blank=True)),
            ('variants_display_type', models.IntegerField(_(u"Variants display type"), default=0)),
            ('parent', models.ForeignKey(orm.Product, related_name="variants", null=True, verbose_name=_(u"Parent"), blank=True)),
            ('active_name', models.BooleanField(_(u"Active name"), default=True)),
            ('active_sku', models.BooleanField(_(u"Active SKU"), default=True)),
            ('active_short_description', models.BooleanField(_(u"Active short description"), default=False)),
            ('active_description', models.BooleanField(_(u"Active description"), default=False)),
            ('active_price', models.BooleanField(_(u"Active Price"), default=True)),
            ('active_images', models.BooleanField(_(u"Active Images"), default=False)),
            ('active_related_products', models.BooleanField(_(u"Active related products"), default=False)),
            ('active_accessories', models.BooleanField(_(u"Active accessories"), default=False)),
            ('active_meta_description', models.BooleanField(_(u"Active meta description"), default=False)),
            ('active_meta_keywords', models.BooleanField(_(u"Active meta keywords"), default=False)),
        ))
        db.send_create_signal('catalog', ['Product'])
        
        # Adding model 'DeliveryTime'
        db.create_table('catalog_deliverytime', (
            ('id', models.AutoField(primary_key=True)),
            ('min', models.FloatField(_(u"Min"))),
            ('max', models.FloatField(_(u"Max"))),
            ('unit', models.PositiveSmallIntegerField(_(u"Unit"), default=2)),
            ('description', models.TextField(_(u"Description"), blank=True)),
        ))
        db.send_create_signal('catalog', ['DeliveryTime'])
        
        # Adding model 'PropertyOption'
        db.create_table('catalog_propertyoption', (
            ('id', models.AutoField(primary_key=True)),
            ('property', models.ForeignKey(orm.Property, related_name="options", verbose_name=_(u"Property"))),
            ('name', models.CharField(_(u"Name"), max_length=30)),
            ('price', models.FloatField(_(u"Price"), default=0.0, null=True, blank=True)),
            ('position', models.IntegerField(_(u"Position"), default=99)),
        ))
        db.send_create_signal('catalog', ['PropertyOption'])
        
        # Adding model 'ProductsPropertiesRelation'
        db.create_table('catalog_productspropertiesrelation', (
            ('id', models.AutoField(primary_key=True)),
            ('product', models.ForeignKey(orm.Product, related_name="productsproperties", verbose_name=_(u"Product"))),
            ('property', models.ForeignKey(orm.Property, verbose_name=_(u"Property"))),
            ('position', models.IntegerField(_(u"Position"), default=999)),
        ))
        db.send_create_signal('catalog', ['ProductsPropertiesRelation'])
        
        # Adding model 'Image'
        db.create_table('catalog_image', (
            ('id', models.AutoField(primary_key=True)),
            ('content_type', models.ForeignKey(orm['contenttypes.ContentType'], related_name="image", null=True, verbose_name=_(u"Content type"), blank=True)),
            ('content_id', models.PositiveIntegerField(_(u"Content id"), null=True, blank=True)),
            ('title', models.CharField(_(u"Title"), max_length=100, blank=True)),
            ('image', ImageWithThumbsField(_(u"Image"), null=True, sizes=((60,60),(100,100),(200,200),(400,400)), blank=True)),
            ('position', models.PositiveSmallIntegerField(_(u"Position"), default=999)),
        ))
        db.send_create_signal('catalog', ['Image'])
        
        # Adding model 'GroupsPropertiesRelation'
        db.create_table('catalog_groupspropertiesrelation', (
            ('id', models.AutoField(primary_key=True)),
            ('group', models.ForeignKey(orm.PropertyGroup, related_name="groupproperties", verbose_name=_(u"Group"))),
            ('property', models.ForeignKey(orm.Property, verbose_name=_(u"Property"))),
            ('position', models.IntegerField(_(u"Position"), default=999)),
        ))
        db.send_create_signal('catalog', ['GroupsPropertiesRelation'])
        
        # Adding model 'Property'
        db.create_table('catalog_property', (
            ('id', models.AutoField(primary_key=True)),
            ('name', models.CharField(_(u"Name"), max_length=50)),
            ('position', models.IntegerField(_(u"Position"), null=True, blank=True)),
            ('unit', models.CharField(_(u"Unit"), max_length=15, blank=True)),
            ('display_on_product', models.BooleanField(_(u"Display on product"), default=True)),
            ('local', models.BooleanField(default=False)),
            ('filterable', models.BooleanField(default=True)),
            ('display_no_results', models.BooleanField(_(u"Display no results"), default=False)),
            ('type', models.PositiveSmallIntegerField(_(u"Type"), default=2)),
            ('step_type', models.PositiveSmallIntegerField(_(u"Step type"), default=1)),
            ('step', models.IntegerField(_(u"Step"), null=True, blank=True)),
        ))
        db.send_create_signal('catalog', ['Property'])
        
        # Adding ManyToManyField 'Product.related_products'
        db.create_table('catalog_product_related_products', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('from_product', models.ForeignKey(orm.Product, null=False)),
            ('to_product', models.ForeignKey(orm.Product, null=False))
        ))
        
        # Adding ManyToManyField 'PropertyGroup.products'
        db.create_table('catalog_propertygroup_products', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('propertygroup', models.ForeignKey(orm.PropertyGroup, null=False)),
            ('product', models.ForeignKey(orm.Product, null=False))
        ))
        
        # Adding ManyToManyField 'Category.products'
        db.create_table('catalog_category_products', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('category', models.ForeignKey(orm.Category, null=False)),
            ('product', models.ForeignKey(orm.Product, null=False))
        ))
        
        # Creating unique_together for [product, property] on ProductsPropertiesRelation.
        db.create_unique('catalog_productspropertiesrelation', ['product_id', 'property_id'])
        
        # Creating unique_together for [product, property, value] on ProductPropertyValue.
        db.create_unique('catalog_productpropertyvalue', ['product_id', 'property_id', 'value'])
        
        # Creating unique_together for [group, property] on GroupsPropertiesRelation.
        db.create_unique('catalog_groupspropertiesrelation', ['group_id', 'property_id'])
        
    
    
    def backwards(self, orm):
        
        # Deleting model 'ProductPropertyValue'
        db.delete_table('catalog_productpropertyvalue')
        
        # Deleting model 'StaticBlock'
        db.delete_table('catalog_staticblock')
        
        # Deleting model 'PropertyGroup'
        db.delete_table('catalog_propertygroup')
        
        # Deleting model 'Category'
        db.delete_table('catalog_category')
        
        # Deleting model 'ProductAccessories'
        db.delete_table('catalog_productaccessories')
        
        # Deleting model 'FilterStep'
        db.delete_table('catalog_filterstep')
        
        # Deleting model 'Product'
        db.delete_table('catalog_product')
        
        # Deleting model 'DeliveryTime'
        db.delete_table('catalog_deliverytime')
        
        # Deleting model 'PropertyOption'
        db.delete_table('catalog_propertyoption')
        
        # Deleting model 'ProductsPropertiesRelation'
        db.delete_table('catalog_productspropertiesrelation')
        
        # Deleting model 'Image'
        db.delete_table('catalog_image')
        
        # Deleting model 'GroupsPropertiesRelation'
        db.delete_table('catalog_groupspropertiesrelation')
        
        # Deleting model 'Property'
        db.delete_table('catalog_property')
        
        # Dropping ManyToManyField 'Product.related_products'
        db.delete_table('catalog_product_related_products')
        
        # Dropping ManyToManyField 'PropertyGroup.products'
        db.delete_table('catalog_propertygroup_products')
        
        # Dropping ManyToManyField 'Category.products'
        db.delete_table('catalog_category_products')
        
        # Deleting unique_together for [product, property] on ProductsPropertiesRelation.
        db.delete_unique('catalog_productspropertiesrelation', ['product_id', 'property_id'])
        
        # Deleting unique_together for [product, property, value] on ProductPropertyValue.
        db.delete_unique('catalog_productpropertyvalue', ['product_id', 'property_id', 'value'])
        
        # Deleting unique_together for [group, property] on GroupsPropertiesRelation.
        db.delete_unique('catalog_groupspropertiesrelation', ['group_id', 'property_id'])
        
    
    
    models = {
        'catalog.productpropertyvalue': {
            'Meta': {'unique_together': '("product","property","value")'},
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'parent_id': ('models.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'product': ('models.ForeignKey', ["orm['catalog.Product']"], {'related_name': '"property_values"', 'verbose_name': '_(u"Product")'}),
            'property': ('models.ForeignKey', ["orm['catalog.Property']"], {'related_name': '"property_values"', 'verbose_name': '_(u"Property")'}),
            'value': ('models.CharField', [], {'max_length': '100', 'blank': 'True'}),
            'value_as_float': ('models.FloatField', [], {'null': 'True', 'blank': 'True'})
        },
        'catalog.staticblock': {
            'html': ('models.TextField', ['_(u"HTML")'], {'blank': 'True'}),
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'name': ('models.CharField', ['_(u"Name")'], {'max_length': '30'})
        },
        'catalog.propertygroup': {
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'name': ('models.CharField', [], {'max_length': '50', 'blank': 'True'}),
            'products': ('models.ManyToManyField', ["orm['catalog.Product']"], {'related_name': '"property_groups"', 'verbose_name': '_(u"Products")'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label','model'),)", 'db_table': "'django_content_type'"},
            '_stub': True,
            'id': ('models.AutoField', [], {'primary_key': 'True'})
        },
        'catalog.category': {
            'Meta': {'ordering': '("position",)'},
            'active_formats': ('models.BooleanField', ['_(u"Active formats")'], {'default': 'False'}),
            'category_cols': ('models.IntegerField', ['_(u"Category cols")'], {'default': '3'}),
            'content': ('models.IntegerField', ['_(u"Content")'], {'default': '1'}),
            'description': ('models.TextField', ['_(u"Description")'], {'blank': 'True'}),
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'image': ('ImageWithThumbsField', ['_(u"Image")'], {'null': 'True', 'sizes': '((60,60),(100,100),(200,200),(400,400))', 'blank': 'True'}),
            'meta_description': ('models.TextField', ['_(u"Meta description")'], {'blank': 'True'}),
            'meta_keywords': ('models.TextField', ['_(u"Meta keywords")'], {'blank': 'True'}),
            'name': ('models.CharField', ['_(u"Name")'], {'max_length': '50'}),
            'parent': ('models.ForeignKey', ["orm['catalog.Category']"], {'null': 'True', 'verbose_name': '_(u"Parent")', 'blank': 'True'}),
            'position': ('models.IntegerField', ['_(u"Position")'], {'default': '1000'}),
            'product_cols': ('models.IntegerField', ['_(u"Product cols")'], {'default': '3'}),
            'product_rows': ('models.IntegerField', ['_(u"Product rows")'], {'default': '3'}),
            'products': ('models.ManyToManyField', ["orm['catalog.Product']"], {'related_name': '"categories"', 'verbose_name': '_(u"Products")', 'blank': 'True'}),
            'short_description': ('models.TextField', ['_(u"Short description")'], {'blank': 'True'}),
            'show_all_products': ('models.BooleanField', ['_(u"Show all products")'], {'default': 'True'}),
            'slug': ('models.SlugField', ['_(u"Slug")'], {'unique': 'True'}),
            'static_block': ('models.ForeignKey', ["orm['catalog.StaticBlock']"], {'related_name': '"categories"', 'null': 'True', 'verbose_name': '_(u"Static block")', 'blank': 'True'})
        },
        'catalog.productaccessories': {
            'Meta': {'ordering': '("position",)'},
            'accessory': ('models.ForeignKey', ["orm['catalog.Product']"], {'related_name': '"productaccessories_accessory"', 'verbose_name': '_(u"Acessory")'}),
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'position': ('models.IntegerField', ['_(u"Position")'], {'default': '999'}),
            'product': ('models.ForeignKey', ["orm['catalog.Product']"], {'related_name': '"productaccessories_product"', 'verbose_name': '_(u"Product")'}),
            'quantity': ('models.FloatField', ['_(u"Quantity")'], {'default': '1'})
        },
        'catalog.filterstep': {
            'Meta': {'ordering': '["start"]'},
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'property': ('models.ForeignKey', ["orm['catalog.Property']"], {'related_name': '"steps"', 'verbose_name': '_(u"Property")'}),
            'start': ('models.FloatField', [], {})
        },
        'catalog.product': {
            'Meta': {'ordering': '("name",)'},
            'accessories': ('models.ManyToManyField', ["orm['catalog.Product']"], {'related_name': '"reverse_accessories"', 'through': '"ProductAccessories"', 'blank': 'True', 'symmetrical': 'False', 'null': 'True', 'verbose_name': '_(u"Acessories")'}),
            'active': ('models.BooleanField', ['_(u"Active")'], {'default': 'False'}),
            'active_accessories': ('models.BooleanField', ['_(u"Active accessories")'], {'default': 'False'}),
            'active_description': ('models.BooleanField', ['_(u"Active description")'], {'default': 'False'}),
            'active_images': ('models.BooleanField', ['_(u"Active Images")'], {'default': 'False'}),
            'active_meta_description': ('models.BooleanField', ['_(u"Active meta description")'], {'default': 'False'}),
            'active_meta_keywords': ('models.BooleanField', ['_(u"Active meta keywords")'], {'default': 'False'}),
            'active_name': ('models.BooleanField', ['_(u"Active name")'], {'default': 'True'}),
            'active_price': ('models.BooleanField', ['_(u"Active Price")'], {'default': 'True'}),
            'active_related_products': ('models.BooleanField', ['_(u"Active related products")'], {'default': 'False'}),
            'active_short_description': ('models.BooleanField', ['_(u"Active short description")'], {'default': 'False'}),
            'active_sku': ('models.BooleanField', ['_(u"Active SKU")'], {'default': 'True'}),
            'creation_date': ('models.DateTimeField', ['_(u"Creation date")'], {'auto_now_add': 'True'}),
            'default_variant': ('models.ForeignKey', ["orm['catalog.Product']"], {'null': 'True', 'verbose_name': '_(u"Default variant")', 'blank': 'True'}),
            'deliverable': ('models.BooleanField', ['_(u"Deliverable")'], {'default': 'True'}),
            'delivery_time': ('models.ForeignKey', ["orm['catalog.DeliveryTime']"], {'related_name': '"products_delivery_time"', 'null': 'True', 'verbose_name': '_(u"Delivery time")', 'blank': 'True'}),
            'description': ('models.TextField', ['_(u"Description")'], {'blank': 'True'}),
            'effective_price': ('models.FloatField', ['_(u"Price")'], {'blank': 'True'}),
            'for_sale': ('models.BooleanField', ['_(u"For sale")'], {'default': 'False'}),
            'for_sale_price': ('models.FloatField', ['_(u"For sale price")'], {'default': '0.0'}),
            'height': ('models.FloatField', ['_(u"Height")'], {'default': '0.0'}),
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'images': ('generic.GenericRelation', ["orm['catalog.Image']"], {'object_id_field': '"content_id"', 'verbose_name': '_(u"Images")', 'content_type_field': '"content_type"'}),
            'length': ('models.FloatField', ['_(u"Length")'], {'default': '0.0'}),
            'manage_stock_amount': ('models.BooleanField', ['_(u"Manage stock amount")'], {'default': 'True'}),
            'manual_delivery_time': ('models.BooleanField', ['_(u"Manual delivery time")'], {'default': 'False'}),
            'meta_description': ('models.TextField', ['_(u"Meta description")'], {'blank': 'True'}),
            'meta_keywords': ('models.TextField', ['_(u"Meta keywords")'], {'blank': 'True'}),
            'name': ('models.CharField', ['_(u"Name")'], {'max_length': '80'}),
            'order_time': ('models.ForeignKey', ["orm['catalog.DeliveryTime']"], {'related_name': '"products_order_time"', 'null': 'True', 'verbose_name': '_(u"Order time")', 'blank': 'True'}),
            'ordered_at': ('models.DateField', ['_(u"Ordered at")'], {'null': 'True', 'blank': 'True'}),
            'parent': ('models.ForeignKey', ["orm['catalog.Product']"], {'related_name': '"variants"', 'null': 'True', 'verbose_name': '_(u"Parent")', 'blank': 'True'}),
            'price': ('models.FloatField', ['_(u"Price")'], {'default': '0.0'}),
            'related_products': ('models.ManyToManyField', ["orm['catalog.Product']"], {'related_name': '"reverse_related_products"', 'symmetrical': 'False', 'null': 'True', 'verbose_name': '_(u"Related products")', 'blank': 'True'}),
            'short_description': ('models.TextField', ['_(u"Short description")'], {'blank': 'True'}),
            'sku': ('models.CharField', ['_(u"SKU")'], {'max_length': '30', 'blank': 'True'}),
            'slug': ('models.SlugField', ['_(u"Slug")'], {'max_length': '80', 'unique': 'True'}),
            'stock_amount': ('models.DecimalField', ['_(u"Stock amount")'], {'default': '0', 'max_digits': '3', 'decimal_places': '1'}),
            'sub_type': ('models.CharField', ['_(u"Subtype")'], {'default': "'0'", 'max_length': '10'}),
            'tax': ('models.ForeignKey', ["orm['tax.Tax']"], {'null': 'True', 'verbose_name': '_(u"Tax")', 'blank': 'True'}),
            'variants_display_type': ('models.IntegerField', ['_(u"Variants display type")'], {'default': '0'}),
            'weight': ('models.FloatField', ['_(u"Weight")'], {'default': '0.0'}),
            'width': ('models.FloatField', ['_(u"Width")'], {'default': '0.0'})
        },
        'catalog.deliverytime': {
            'Meta': {'ordering': '("min",)'},
            'description': ('models.TextField', ['_(u"Description")'], {'blank': 'True'}),
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'max': ('models.FloatField', ['_(u"Max")'], {}),
            'min': ('models.FloatField', ['_(u"Min")'], {}),
            'unit': ('models.PositiveSmallIntegerField', ['_(u"Unit")'], {'default': '2'})
        },
        'catalog.propertyoption': {
            'Meta': {'ordering': '["position"]'},
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'name': ('models.CharField', ['_(u"Name")'], {'max_length': '30'}),
            'position': ('models.IntegerField', ['_(u"Position")'], {'default': '99'}),
            'price': ('models.FloatField', ['_(u"Price")'], {'default': '0.0', 'null': 'True', 'blank': 'True'}),
            'property': ('models.ForeignKey', ["orm['catalog.Property']"], {'related_name': '"options"', 'verbose_name': '_(u"Property")'})
        },
        'catalog.productspropertiesrelation': {
            'Meta': {'ordering': '("position",)', 'unique_together': '("product","property")'},
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'position': ('models.IntegerField', ['_(u"Position")'], {'default': '999'}),
            'product': ('models.ForeignKey', ["orm['catalog.Product']"], {'related_name': '"productsproperties"', 'verbose_name': '_(u"Product")'}),
            'property': ('models.ForeignKey', ["orm['catalog.Property']"], {'verbose_name': '_(u"Property")'})
        },
        'catalog.image': {
            'Meta': {'ordering': '("position",)'},
            'content_id': ('models.PositiveIntegerField', ['_(u"Content id")'], {'null': 'True', 'blank': 'True'}),
            'content_type': ('models.ForeignKey', ["orm['contenttypes.ContentType']"], {'related_name': '"image"', 'null': 'True', 'verbose_name': '_(u"Content type")', 'blank': 'True'}),
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'image': ('ImageWithThumbsField', ['_(u"Image")'], {'null': 'True', 'sizes': '((60,60),(100,100),(200,200),(400,400))', 'blank': 'True'}),
            'position': ('models.PositiveSmallIntegerField', ['_(u"Position")'], {'default': '999'}),
            'title': ('models.CharField', ['_(u"Title")'], {'max_length': '100', 'blank': 'True'})
        },
        'catalog.groupspropertiesrelation': {
            'Meta': {'ordering': '("position",)', 'unique_together': '("group","property")'},
            'group': ('models.ForeignKey', ["orm['catalog.PropertyGroup']"], {'related_name': '"groupproperties"', 'verbose_name': '_(u"Group")'}),
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'position': ('models.IntegerField', ['_(u"Position")'], {'default': '999'}),
            'property': ('models.ForeignKey', ["orm['catalog.Property']"], {'verbose_name': '_(u"Property")'})
        },
        'tax.tax': {
            '_stub': True,
            'id': ('models.AutoField', [], {'primary_key': 'True'})
        },
        'catalog.property': {
            'Meta': {'ordering': '["position"]'},
            'display_no_results': ('models.BooleanField', ['_(u"Display no results")'], {'default': 'False'}),
            'display_on_product': ('models.BooleanField', ['_(u"Display on product")'], {'default': 'True'}),
            'filterable': ('models.BooleanField', [], {'default': 'True'}),
            'groups': ('models.ManyToManyField', ["orm['catalog.PropertyGroup']"], {'related_name': '"properties"', 'through': '"GroupsPropertiesRelation"', 'null': 'True', 'verbose_name': '_(u"Group")', 'blank': 'True'}),
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'local': ('models.BooleanField', [], {'default': 'False'}),
            'name': ('models.CharField', ['_(u"Name")'], {'max_length': '50'}),
            'position': ('models.IntegerField', ['_(u"Position")'], {'null': 'True', 'blank': 'True'}),
            'products': ('models.ManyToManyField', ["orm['catalog.Product']"], {'related_name': '"properties"', 'through': '"ProductsPropertiesRelation"', 'null': 'True', 'verbose_name': '_(u"Products")', 'blank': 'True'}),
            'step': ('models.IntegerField', ['_(u"Step")'], {'null': 'True', 'blank': 'True'}),
            'step_type': ('models.PositiveSmallIntegerField', ['_(u"Step type")'], {'default': '1'}),
            'type': ('models.PositiveSmallIntegerField', ['_(u"Type")'], {'default': '2'}),
            'unit': ('models.CharField', ['_(u"Unit")'], {'max_length': '15', 'blank': 'True'})
        }
    }
    
    complete_apps = ['catalog']




from south.db import db
from django.db import models
from lfs.customer.models import *

class Migration:
    
    def forwards(self, orm):
        
        # Adding model 'CreditCard'
        db.create_table('customer_creditcard', (
            ('customer', models.ForeignKey(orm.Customer, related_name="credit_cards", null=True, verbose_name=_(u"Customer"), blank=True)),
            ('number', models.CharField(_(u"Number"), max_length=30, blank=True)),
            ('expiration_date_year', models.IntegerField(_(u"Expiration date year"))),
            ('expiration_date_month', models.IntegerField(_(u"Expiration date month"))),
            ('owner', models.CharField(_(u"Owner"), max_length=100, blank=True)),
            ('type', models.CharField(_(u"Type"), max_length=30, blank=True)),
            ('id', models.AutoField(primary_key=True)),
        ))
        db.send_create_signal('customer', ['CreditCard'])
        
        # Adding model 'BankAccount'
        db.create_table('customer_bankaccount', (
            ('customer', models.ForeignKey(orm.Customer, related_name="bank_accounts", null=True, verbose_name=_(u"Customer"), blank=True)),
            ('bank_name', models.CharField(_(u"Bank name"), max_length=100, blank=True)),
            ('bank_identification_code', models.CharField(_(u"Bank identification code"), max_length=30, blank=True)),
            ('depositor', models.CharField(_(u"Depositor"), max_length=100, blank=True)),
            ('account_number', models.CharField(_(u"Account number"), max_length=30, blank=True)),
            ('id', models.AutoField(primary_key=True)),
        ))
        db.send_create_signal('customer', ['BankAccount'])
        
        # Adding model 'Address'
        db.create_table('customer_address', (
            ('customer', models.ForeignKey(orm.Customer, related_name="addresses", null=True, verbose_name=_(u"Customer"), blank=True)),
            ('city', models.CharField(_("City"), max_length=50)),
            ('firstname', models.CharField(_("Firstname"), max_length=50)),
            ('lastname', models.CharField(_("Lastname"), max_length=50)),
            ('email', models.EmailField(_("E-Mail"), max_length=50, null=True, blank=True)),
            ('phone', models.CharField(_("Phone"), max_length=20, blank=True)),
            ('street', models.CharField(_("Street"), max_length=100)),
            ('country', models.ForeignKey(orm['core.Country'], verbose_name=_("Country"))),
            ('id', models.AutoField(primary_key=True)),
            ('zip_code', models.CharField(_("Zip code"), max_length=10)),
        ))
        db.send_create_signal('customer', ['Address'])
        
        # Adding model 'Customer'
        db.create_table('customer_customer', (
            ('selected_shipping_method', models.ForeignKey(orm['shipping.ShippingMethod'], related_name="selected_shipping_method", null=True, verbose_name=_(u"Selected shipping method"), blank=True)),
            ('selected_bank_account', models.ForeignKey(orm.BankAccount, related_name="selected_bank_account", null=True, verbose_name=_(u"Bank account"), blank=True)),
            ('selected_invoice_address', models.ForeignKey(orm.Address, related_name="selected_invoice_address", null=True, verbose_name=_(u"Selected invoice address"), blank=True)),
            ('selected_shipping_address', models.ForeignKey(orm.Address, related_name="selected_shipping_address", null=True, verbose_name=_(u"Selected shipping address"), blank=True)),
            ('session', models.CharField(max_length=100, blank=True)),
            ('user', models.ForeignKey(orm['auth.User'], null=True, blank=True)),
            ('selected_payment_method', models.ForeignKey(orm['payment.PaymentMethod'], related_name="selected_payment_method", null=True, verbose_name=_(u"Selected payment method"), blank=True)),
            ('selected_country', models.ForeignKey(orm['core.Country'], null=True, verbose_name=_(u"Selected country"), blank=True)),
            ('id', models.AutoField(primary_key=True)),
            ('selected_credit_card', models.ForeignKey(orm.CreditCard, related_name="selected_credit_card", null=True, verbose_name=_(u"Credit card"), blank=True)),
        ))
        db.send_create_signal('customer', ['Customer'])
        
    
    
    def backwards(self, orm):
        
        # Deleting model 'CreditCard'
        db.delete_table('customer_creditcard')
        
        # Deleting model 'BankAccount'
        db.delete_table('customer_bankaccount')
        
        # Deleting model 'Address'
        db.delete_table('customer_address')
        
        # Deleting model 'Customer'
        db.delete_table('customer_customer')
        
    
    
    models = {
        'customer.creditcard': {
            'customer': ('models.ForeignKey', ['Customer'], {'related_name': '"credit_cards"', 'null': 'True', 'verbose_name': '_(u"Customer")', 'blank': 'True'}),
            'expiration_date_month': ('models.IntegerField', ['_(u"Expiration date month")'], {}),
            'expiration_date_year': ('models.IntegerField', ['_(u"Expiration date year")'], {}),
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'number': ('models.CharField', ['_(u"Number")'], {'max_length': '30', 'blank': 'True'}),
            'owner': ('models.CharField', ['_(u"Owner")'], {'max_length': '100', 'blank': 'True'}),
            'type': ('models.CharField', ['_(u"Type")'], {'max_length': '30', 'blank': 'True'})
        },
        'payment.paymentmethod': {
            'Meta': {'ordering': '("priority",)'},
            '_stub': True,
            'id': ('models.AutoField', [], {'primary_key': 'True'})
        },
        'customer.customer': {
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'selected_bank_account': ('models.ForeignKey', ['"BankAccount"'], {'related_name': '"selected_bank_account"', 'null': 'True', 'verbose_name': '_(u"Bank account")', 'blank': 'True'}),
            'selected_country': ('models.ForeignKey', ['Country'], {'null': 'True', 'verbose_name': '_(u"Selected country")', 'blank': 'True'}),
            'selected_credit_card': ('models.ForeignKey', ['"CreditCard"'], {'related_name': '"selected_credit_card"', 'null': 'True', 'verbose_name': '_(u"Credit card")', 'blank': 'True'}),
            'selected_invoice_address': ('models.ForeignKey', ['"Address"'], {'related_name': '"selected_invoice_address"', 'null': 'True', 'verbose_name': '_(u"Selected invoice address")', 'blank': 'True'}),
            'selected_payment_method': ('models.ForeignKey', ['PaymentMethod'], {'related_name': '"selected_payment_method"', 'null': 'True', 'verbose_name': '_(u"Selected payment method")', 'blank': 'True'}),
            'selected_shipping_address': ('models.ForeignKey', ['"Address"'], {'related_name': '"selected_shipping_address"', 'null': 'True', 'verbose_name': '_(u"Selected shipping address")', 'blank': 'True'}),
            'selected_shipping_method': ('models.ForeignKey', ['ShippingMethod'], {'related_name': '"selected_shipping_method"', 'null': 'True', 'verbose_name': '_(u"Selected shipping method")', 'blank': 'True'}),
            'session': ('models.CharField', [], {'max_length': '100', 'blank': 'True'}),
            'user': ('models.ForeignKey', ['User'], {'null': 'True', 'blank': 'True'})
        },
        'auth.user': {
            '_stub': True,
            'id': ('models.AutoField', [], {'primary_key': 'True'})
        },
        'customer.address': {
            'city': ('models.CharField', ['_("City")'], {'max_length': '50'}),
            'country': ('models.ForeignKey', ['Country'], {'verbose_name': '_("Country")'}),
            'customer': ('models.ForeignKey', ['Customer'], {'related_name': '"addresses"', 'null': 'True', 'verbose_name': '_(u"Customer")', 'blank': 'True'}),
            'email': ('models.EmailField', ['_("E-Mail")'], {'max_length': '50', 'null': 'True', 'blank': 'True'}),
            'firstname': ('models.CharField', ['_("Firstname")'], {'max_length': '50'}),
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'lastname': ('models.CharField', ['_("Lastname")'], {'max_length': '50'}),
            'phone': ('models.CharField', ['_("Phone")'], {'max_length': '20', 'blank': 'True'}),
            'street': ('models.CharField', ['_("Street")'], {'max_length': '100'}),
            'zip_code': ('models.CharField', ['_("Zip code")'], {'max_length': '10'})
        },
        'shipping.shippingmethod': {
            'Meta': {'ordering': '("priority",)'},
            '_stub': True,
            'id': ('models.AutoField', [], {'primary_key': 'True'})
        },
        'customer.bankaccount': {
            'account_number': ('models.CharField', ['_(u"Account number")'], {'max_length': '30', 'blank': 'True'}),
            'bank_identification_code': ('models.CharField', ['_(u"Bank identification code")'], {'max_length': '30', 'blank': 'True'}),
            'bank_name': ('models.CharField', ['_(u"Bank name")'], {'max_length': '100', 'blank': 'True'}),
            'customer': ('models.ForeignKey', ['Customer'], {'related_name': '"bank_accounts"', 'null': 'True', 'verbose_name': '_(u"Customer")', 'blank': 'True'}),
            'depositor': ('models.CharField', ['_(u"Depositor")'], {'max_length': '100', 'blank': 'True'}),
            'id': ('models.AutoField', [], {'primary_key': 'True'})
        },
        'core.country': {
            'Meta': {'ordering': '("name",)'},
            '_stub': True,
            'id': ('models.AutoField', [], {'primary_key': 'True'})
        }
    }
    
    complete_apps = ['customer']



# django imports
from django.db import models
from django.contrib.auth.models import User
from django.utils.translation import ugettext_lazy as _

# lfs imports
from lfs.core.models import Country
from lfs.catalog.models import Product
from lfs.order.settings import ORDER_STATES
from lfs.order.settings import SUBMITTED
from lfs.shipping.models import ShippingMethod
from lfs.payment.models import PaymentMethod
from lfs.payment.settings import PAYPAL
import lfs.payment.utils

# other imports
import uuid


def get_unique_id_str():
    return str(uuid.uuid4())

class Order(models.Model):
    """An order is created when products have been sold.
    """
    user = models.ForeignKey(User, verbose_name=_(u"User"), blank=True, null=True)
    session = models.CharField(_(u"Session"), blank=True, max_length=100)

    created = models.DateTimeField(_(u"Created"), auto_now_add=True)

    state = models.PositiveSmallIntegerField(_(u"State"), choices=ORDER_STATES, default=SUBMITTED)
    state_modified = models.DateTimeField(_(u"State modified"), auto_now_add=True)

    price = models.FloatField(_(u"Price"), default=0.0)
    tax = models.FloatField(_(u"Tax"), default=0.0)

    customer_firstname = models.CharField(_(u"firstname"), max_length=50)
    customer_lastname = models.CharField(_(u"lastname"), max_length=50)
    customer_email = models.CharField(_(u"lastname"), max_length=50)

    invoice_firstname = models.CharField(_(u"Invoice firstname"), max_length=50)
    invoice_lastname = models.CharField(_(u"Invoice lastname"), max_length=50)
    invoice_company_name = models.CharField(_(u"Invoice company name"), max_length=50)
    invoice_street = models.CharField(_(u"Invoice street"), blank=True, max_length=100)
    invoice_zip_code = models.CharField(_(u"Invoice zip code"), max_length=10)
    invoice_city = models.CharField(_(u"Invoice city"), max_length=50)
    invoice_state = models.CharField(_(u"Invoice state"), max_length=50)
    invoice_country = models.ForeignKey(Country, related_name="orders_invoice_country")
    invoice_phone = models.CharField(_(u"Invoice phone"), blank=True, max_length=20)

    shipping_firstname = models.CharField(_(u"Shipping firstname"), max_length=50)
    shipping_lastname = models.CharField(_(u"Shipping lastname"), max_length=50)
    shipping_company_name = models.CharField(_(u"Shipping company name"), max_length=50)
    shipping_street = models.CharField(_(u"Shipping street"), blank=True, max_length=100)
    shipping_zip_code = models.CharField(_(u"Shipping zip code"), max_length=10)
    shipping_city = models.CharField(_(u"Shipping city"), max_length=50)
    shipping_state = models.CharField(_(u"Shipping state"), max_length=50)
    shipping_country = models.ForeignKey(Country, related_name="orders_shipping_country")
    shipping_phone = models.CharField(_(u"Shipping phone"), blank=True, max_length=20)

    shipping_method = models.ForeignKey(ShippingMethod, verbose_name=_(u"Shipping Method"), blank=True, null=True)
    shipping_price = models.FloatField(_(u"Shipping Price"), default=0.0)
    shipping_tax = models.FloatField(_(u"Shipping Tax"), default=0.0)

    payment_method = models.ForeignKey(PaymentMethod, verbose_name=_(u"Payment Method"), blank=True, null=True)
    payment_price = models.FloatField(_(u"Payment Price"), default=0.0)
    payment_tax = models.FloatField(_(u"Payment Tax"), default=0.0)

    account_number = models.CharField(_(u"Account number"), blank=True, max_length=30)
    bank_identification_code = models.CharField(_(u"Bank identication code"), blank=True, max_length=30)
    bank_name = models.CharField(_(u"Bank name"), blank=True, max_length=100)
    depositor = models.CharField(_(u"Depositor"), blank=True, max_length=100)

    message = models.TextField(_(u"Message"), blank=True)

    uuid = models.CharField(max_length=50, editable=False,unique=True, default=get_unique_id_str)

    class Meta:
        ordering = ("-created", )

    def __unicode__(self):
        return "%s (%s %s)" % (self.created.strftime("%x %X"), self.customer_firstname, self.customer_lastname)

    def get_pay_link(self):
        """Returns a pay link for the selected payment method.
        """
        if self.payment_method.id == PAYPAL:
            return lfs.payment.utils.create_paypal_link_for_order(self)

        return None

    def get_name(self):
        order_name = ""
        for order_item in self.items.all():
            if order_item.product is not None:
                order_name = order_name + order_item.product.get_name() + ", "

        order_name.strip(', ')
        return order_name

class OrderItem(models.Model):
    """An order items holds the sold product, its amount and some other relevant
    product values like the price at the time the product has been sold.
    """
    order = models.ForeignKey(Order, related_name="items")

    price_net = models.FloatField(_(u"Price net"), default=0.0)
    price_gross = models.FloatField(_(u"Price gross"), default=0.0)
    tax = models.FloatField(_(u"Tax"), default=0.0)

    # A optional reference to the origin product. This is optional in case the
    # product has been deleted. TODO: Decide: Are products able to be delete?
    product = models.ForeignKey(Product, blank=True, null=True)

    # Values of the product at the time the orders has been created
    product_amount = models.IntegerField(_(u"Product quantity"), blank=True, null=True)
    product_sku = models.CharField(_(u"Product SKU"), blank=True, max_length=100)
    product_name = models.CharField(_(u"Product name"), blank=True, max_length=100)
    product_price_net = models.FloatField(_(u"Product price net"), default=0.0)
    product_price_gross = models.FloatField(_(u"Product price gross"), default=0.0)
    product_tax = models.FloatField(_(u"Product tax"), default=0.0)

    def __unicode__(self):
        return "%s" % self.product_name



from south.db import db
from django.db import models
from lfs.payment.models import *

class Migration:
    
    def forwards(self, orm):
        
        # Adding model 'PaymentMethod'
        db.create_table('payment_paymentmethod', (
            ('id', models.AutoField(primary_key=True)),
            ('name', models.CharField(_(u"Name"), max_length=50, blank=True)),
            ('active', models.BooleanField(_(u"Active"), default=False)),
            ('description', models.TextField(_(u"Description"), blank=True)),
            ('note', models.TextField(_(u"note"), blank=True)),
            ('priority', models.IntegerField(_(u"Priority"), default=0)),
            ('image', models.ImageField(_(u"Image"), null=True, blank=True)),
            ('tax', models.ForeignKey(orm['tax.Tax'], null=True, verbose_name=_(u"Tax"), blank=True)),
            ('price', models.FloatField(_(u"Price"), default=0.0)),
            ('deletable', models.BooleanField(default=True)),
        ))
        db.send_create_signal('payment', ['PaymentMethod'])
        
        # Adding model 'PaymentMethodPrice'
        db.create_table('payment_paymentmethodprice', (
            ('id', models.AutoField(primary_key=True)),
            ('payment_method', models.ForeignKey(orm.PaymentMethod, related_name="prices", verbose_name=_(u"Payment method"))),
            ('price', models.FloatField(_(u"Price"), default=0.0)),
            ('priority', models.IntegerField(_(u"Priority"), default=0)),
            ('active', models.BooleanField(_(u"Active"), default=False)),
        ))
        db.send_create_signal('payment', ['PaymentMethodPrice'])
        
    
    
    def backwards(self, orm):
        
        # Deleting model 'PaymentMethod'
        db.delete_table('payment_paymentmethod')
        
        # Deleting model 'PaymentMethodPrice'
        db.delete_table('payment_paymentmethodprice')
        
    
    
    models = {
        'payment.paymentmethod': {
            'Meta': {'ordering': '("priority",)'},
            'active': ('models.BooleanField', ['_(u"Active")'], {'default': 'False'}),
            'deletable': ('models.BooleanField', [], {'default': 'True'}),
            'description': ('models.TextField', ['_(u"Description")'], {'blank': 'True'}),
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'image': ('models.ImageField', ['_(u"Image")'], {'null': 'True', 'blank': 'True'}),
            'name': ('models.CharField', ['_(u"Name")'], {'max_length': '50', 'blank': 'True'}),
            'note': ('models.TextField', ['_(u"note")'], {'blank': 'True'}),
            'price': ('models.FloatField', ['_(u"Price")'], {'default': '0.0'}),
            'priority': ('models.IntegerField', ['_(u"Priority")'], {'default': '0'}),
            'tax': ('models.ForeignKey', ["orm['tax.Tax']"], {'null': 'True', 'verbose_name': '_(u"Tax")', 'blank': 'True'})
        },
        'tax.tax': {
            '_stub': True,
            'id': ('models.AutoField', [], {'primary_key': 'True'})
        },
        'payment.paymentmethodprice': {
            'Meta': {'ordering': '("priority",)'},
            'active': ('models.BooleanField', ['_(u"Active")'], {'default': 'False'}),
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'payment_method': ('models.ForeignKey', ["orm['payment.PaymentMethod']"], {'related_name': '"prices"', 'verbose_name': '_(u"Payment method")'}),
            'price': ('models.FloatField', ['_(u"Price")'], {'default': '0.0'}),
            'priority': ('models.IntegerField', ['_(u"Priority")'], {'default': '0'})
        }
    }
    
    complete_apps = ['payment']




from south.db import db
from django.db import models
from lfs.order.models import *

class Migration:
    
    def forwards(self, orm):
        
        # Adding model 'OrderItem'
        db.create_table('order_orderitem', (
            ('product', models.ForeignKey(orm['catalog.Product'], null=True, blank=True)),
            ('product_name', models.CharField(_(u"Product name"), max_length=100, blank=True)),
            ('product_amount', models.IntegerField(_(u"Product quantity"), null=True, blank=True)),
            ('tax', models.FloatField(_(u"Tax"), default=0.0)),
            ('id', models.AutoField(primary_key=True)),
            ('product_price_net', models.FloatField(_(u"Product price net"), default=0.0)),
            ('product_tax', models.FloatField(_(u"Product tax"), default=0.0)),
            ('product_price_gross', models.FloatField(_(u"Product price gross"), default=0.0)),
            ('product_sku', models.CharField(_(u"Product SKU"), max_length=100, blank=True)),
            ('price_net', models.FloatField(_(u"Price net"), default=0.0)),
            ('order', models.ForeignKey(orm.Order, related_name="items")),
            ('price_gross', models.FloatField(_(u"Price gross"), default=0.0)),
        ))
        db.send_create_signal('order', ['OrderItem'])
        
        # Adding model 'Order'
        db.create_table('order_order', (
            ('shipping_method', models.ForeignKey(orm['shipping.ShippingMethod'], null=True, verbose_name=_(u"Shipping Method"), blank=True)),
            ('shipping_street', models.CharField(_(u"Shipping street"), max_length=100, blank=True)),
            ('payment_method', models.ForeignKey(orm['payment.PaymentMethod'], null=True, verbose_name=_(u"Payment Method"), blank=True)),
            ('bank_identification_code', models.CharField(_(u"Bank identication code"), max_length=30, blank=True)),
            ('customer_firstname', models.CharField(_(u"firstname"), max_length=50)),
            ('shipping_zip_code', models.CharField(_(u"Shipping zip code"), max_length=10)),
            ('depositor', models.CharField(_(u"Depositor"), max_length=100, blank=True)),
            ('session', models.CharField(_(u"Session"), max_length=100, blank=True)),
            ('invoice_city', models.CharField(_(u"Invoice city"), max_length=50)),
            ('shipping_firstname', models.CharField(_(u"Shipping firstname"), max_length=50)),
            ('shipping_tax', models.FloatField(_(u"Shipping Tax"), default=0.0)),
            ('id', models.AutoField(primary_key=True)),
            ('shipping_price', models.FloatField(_(u"Shipping Price"), default=0.0)),
            ('invoice_country', models.ForeignKey(orm['core.Country'], related_name="orders_invoice_country")),
            ('shipping_phone', models.CharField(_(u"Shipping phone"), max_length=20, blank=True)),
            ('payment_tax', models.FloatField(_(u"Payment Tax"), default=0.0)),
            ('state', models.PositiveSmallIntegerField(_(u"State"), default=SUBMITTED)),
            ('invoice_zip_code', models.CharField(_(u"Invoice zip code"), max_length=10)),
            ('customer_lastname', models.CharField(_(u"lastname"), max_length=50)),
            ('shipping_lastname', models.CharField(_(u"Shipping lastname"), max_length=50)),
            ('invoice_street', models.CharField(_(u"Invoice street"), max_length=100, blank=True)),
            ('bank_name', models.CharField(_(u"Bank name"), max_length=100, blank=True)),
            ('customer_email', models.CharField(_(u"lastname"), max_length=50)),
            ('shipping_city', models.CharField(_(u"Shipping city"), max_length=50)),
            ('tax', models.FloatField(_(u"Tax"), default=0.0)),
            ('price', models.FloatField(_(u"Price"), default=0.0)),
            ('account_number', models.CharField(_(u"Account number"), max_length=30, blank=True)),
            ('invoice_phone', models.CharField(_(u"Invoice phone"), max_length=20, blank=True)),
            ('user', models.ForeignKey(orm['auth.User'], null=True, verbose_name=_(u"User"), blank=True)),
            ('invoice_firstname', models.CharField(_(u"Invoice firstname"), max_length=50)),
            ('created', models.DateTimeField(_(u"Created"), auto_now_add=True)),
            ('payment_price', models.FloatField(_(u"Payment Price"), default=0.0)),
            ('message', models.TextField(_(u"Message"), blank=True)),
            ('invoice_lastname', models.CharField(_(u"Invoice lastname"), max_length=50)),
            ('shipping_country', models.ForeignKey(orm['core.Country'], related_name="orders_shipping_country")),
        ))
        db.send_create_signal('order', ['Order'])
        
    
    
    def backwards(self, orm):
        
        # Deleting model 'OrderItem'
        db.delete_table('order_orderitem')
        
        # Deleting model 'Order'
        db.delete_table('order_order')
        
    
    
    models = {
        'order.orderitem': {
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'order': ('models.ForeignKey', ['Order'], {'related_name': '"items"'}),
            'price_gross': ('models.FloatField', ['_(u"Price gross")'], {'default': '0.0'}),
            'price_net': ('models.FloatField', ['_(u"Price net")'], {'default': '0.0'}),
            'product': ('models.ForeignKey', ['Product'], {'null': 'True', 'blank': 'True'}),
            'product_amount': ('models.IntegerField', ['_(u"Product quantity")'], {'null': 'True', 'blank': 'True'}),
            'product_name': ('models.CharField', ['_(u"Product name")'], {'max_length': '100', 'blank': 'True'}),
            'product_price_gross': ('models.FloatField', ['_(u"Product price gross")'], {'default': '0.0'}),
            'product_price_net': ('models.FloatField', ['_(u"Product price net")'], {'default': '0.0'}),
            'product_sku': ('models.CharField', ['_(u"Product SKU")'], {'max_length': '100', 'blank': 'True'}),
            'product_tax': ('models.FloatField', ['_(u"Product tax")'], {'default': '0.0'}),
            'tax': ('models.FloatField', ['_(u"Tax")'], {'default': '0.0'})
        },
        'auth.user': {
            '_stub': True,
            'id': ('models.AutoField', [], {'primary_key': 'True'})
        },
        'shipping.shippingmethod': {
            'Meta': {'ordering': '("priority",)'},
            '_stub': True,
            'id': ('models.AutoField', [], {'primary_key': 'True'})
        },
        'catalog.product': {
            'Meta': {'ordering': '("name",)'},
            '_stub': True,
            'id': ('models.AutoField', [], {'primary_key': 'True'})
        },
        'order.order': {
            'Meta': {'ordering': '("-created",)'},
            'account_number': ('models.CharField', ['_(u"Account number")'], {'max_length': '30', 'blank': 'True'}),
            'bank_identification_code': ('models.CharField', ['_(u"Bank identication code")'], {'max_length': '30', 'blank': 'True'}),
            'bank_name': ('models.CharField', ['_(u"Bank name")'], {'max_length': '100', 'blank': 'True'}),
            'created': ('models.DateTimeField', ['_(u"Created")'], {'auto_now_add': 'True'}),
            'customer_email': ('models.CharField', ['_(u"lastname")'], {'max_length': '50'}),
            'customer_firstname': ('models.CharField', ['_(u"firstname")'], {'max_length': '50'}),
            'customer_lastname': ('models.CharField', ['_(u"lastname")'], {'max_length': '50'}),
            'depositor': ('models.CharField', ['_(u"Depositor")'], {'max_length': '100', 'blank': 'True'}),
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'invoice_city': ('models.CharField', ['_(u"Invoice city")'], {'max_length': '50'}),
            'invoice_country': ('models.ForeignKey', ['Country'], {'related_name': '"orders_invoice_country"'}),
            'invoice_firstname': ('models.CharField', ['_(u"Invoice firstname")'], {'max_length': '50'}),
            'invoice_lastname': ('models.CharField', ['_(u"Invoice lastname")'], {'max_length': '50'}),
            'invoice_phone': ('models.CharField', ['_(u"Invoice phone")'], {'max_length': '20', 'blank': 'True'}),
            'invoice_street': ('models.CharField', ['_(u"Invoice street")'], {'max_length': '100', 'blank': 'True'}),
            'invoice_zip_code': ('models.CharField', ['_(u"Invoice zip code")'], {'max_length': '10'}),
            'message': ('models.TextField', ['_(u"Message")'], {'blank': 'True'}),
            'payment_method': ('models.ForeignKey', ['PaymentMethod'], {'null': 'True', 'verbose_name': '_(u"Payment Method")', 'blank': 'True'}),
            'payment_price': ('models.FloatField', ['_(u"Payment Price")'], {'default': '0.0'}),
            'payment_tax': ('models.FloatField', ['_(u"Payment Tax")'], {'default': '0.0'}),
            'price': ('models.FloatField', ['_(u"Price")'], {'default': '0.0'}),
            'session': ('models.CharField', ['_(u"Session")'], {'max_length': '100', 'blank': 'True'}),
            'shipping_city': ('models.CharField', ['_(u"Shipping city")'], {'max_length': '50'}),
            'shipping_country': ('models.ForeignKey', ['Country'], {'related_name': '"orders_shipping_country"'}),
            'shipping_firstname': ('models.CharField', ['_(u"Shipping firstname")'], {'max_length': '50'}),
            'shipping_lastname': ('models.CharField', ['_(u"Shipping lastname")'], {'max_length': '50'}),
            'shipping_method': ('models.ForeignKey', ['ShippingMethod'], {'null': 'True', 'verbose_name': '_(u"Shipping Method")', 'blank': 'True'}),
            'shipping_phone': ('models.CharField', ['_(u"Shipping phone")'], {'max_length': '20', 'blank': 'True'}),
            'shipping_price': ('models.FloatField', ['_(u"Shipping Price")'], {'default': '0.0'}),
            'shipping_street': ('models.CharField', ['_(u"Shipping street")'], {'max_length': '100', 'blank': 'True'}),
            'shipping_tax': ('models.FloatField', ['_(u"Shipping Tax")'], {'default': '0.0'}),
            'shipping_zip_code': ('models.CharField', ['_(u"Shipping zip code")'], {'max_length': '10'}),
            'state': ('models.PositiveSmallIntegerField', ['_(u"State")'], {'default': 'SUBMITTED'}),
            'tax': ('models.FloatField', ['_(u"Tax")'], {'default': '0.0'}),
            'user': ('models.ForeignKey', ['User'], {'null': 'True', 'verbose_name': '_(u"User")', 'blank': 'True'})
        },
        'payment.paymentmethod': {
            'Meta': {'ordering': '("priority",)'},
            '_stub': True,
            'id': ('models.AutoField', [], {'primary_key': 'True'})
        },
        'core.country': {
            'Meta': {'ordering': '("name",)'},
            '_stub': True,
            'id': ('models.AutoField', [], {'primary_key': 'True'})
        }
    }
    
    complete_apps = ['order']



# django imports
from django.utils.translation import ugettext_lazy as _

DIRECT_DEBIT = 1
CASH_ON_DELIVERY = 2
PAYPAL = 3
PRE_PAYMENT = 4
BY_INVOICE = 5
CREDIT_CARD = 6

CC_AMERICAN_EXPRESS = "AMEX"
CC_MASTERCARD = "MASTER"
CC_VISA = "VISA"

CREDIT_CARD_TYPE_CHOICES = (
    (CC_MASTERCARD, _(u"Mastercard")),
    (CC_VISA, _(u"Visacard")),
)


# python imports
import re

# django imports
from django.db import models
from django.utils.translation import ugettext_lazy as _

# lfs imports
from lfs.checkout.settings import CHECKOUT_TYPES
from lfs.checkout.settings import CHECKOUT_TYPE_SELECT
from lfs.core.fields.thumbs import ImageWithThumbsField
from lfs.core.settings import ACTION_PLACE_CHOICES
from lfs.core.settings import ACTION_PLACE_TABS
from lfs.catalog.models import StaticBlock

class Country(models.Model):
    """Holds country relevant data for the shop.
    """
    code = models.CharField(_(u"Country code"), max_length=2)
    name = models.CharField(_(u"Name"), max_length=100)

    def __unicode__(self):
        return self.name

    class Meta:
        verbose_name_plural = 'Countries'
        ordering = ("name", )

class Action(models.Model):
    """A action is a link which can be displayed on several parts of the web
    page.

    Instance variables:

    - title
      The title of the menu tab
    - link
      The link to the object
    - active
      If true the tab is displayed.
    - position
      the position of the tab within the menu.
    - parent
      Parent tab to create a tree
    """
    title = models.CharField(_(u"Title"), max_length=40)
    link = models.CharField(_(u"Link"), blank=True, max_length=100)
    active = models.BooleanField(_(u"Active"), default=False)
    place = models.PositiveSmallIntegerField(blank=True, null=True, choices=ACTION_PLACE_CHOICES, default=ACTION_PLACE_TABS)
    position = models.IntegerField(_(u"Position"), default=999)
    parent = models.ForeignKey("self", verbose_name=_(u"Parent"), blank=True, null=True)

    def __unicode__(self):
        return self.title

    class Meta:
        ordering=("position",)

class Shop(models.Model):
    """Holds all shop related information.

    At the moment there must be exactly one shop with id == 1. (This may be
    changed in future to provide multi shops.)

    Instance variables:

    - name
       The name of the shop. This is used for the the title of the HTML pages

    - shop_owner
       The shop owner. This is displayed within several places for instance the
       checkout page

    - from_email
       This e-mail address is used for the from header of all outgoing e-mails

    - notification_emails
       This e-mail addresses are used for incoming notification e-mails, e.g.
       received an order. One e-mail address per line.

    - description
       A description of the shop

    - image
      An image which can be used as default image if a category doesn't have one

    - product_cols, product_rows, category_cols
       Upmost format information, which defines how products and categories are
       displayed within several views. These may be inherited by categories and
       sub categories.

    - google_analytics_id
       Used to generate google analytics tracker code and e-commerce code. the
       id has the format UA-xxxxxxx-xx and is provided by Google.

    - ga_site_tracking
       If selected and the google_analytics_id is given google analytics site
       tracking code is inserted into the HTML source code.

    - ga_ecommerce_tracking
       If selected and the google_analytics_id is given google analytics
       e-commerce tracking code is inserted into the HTML source code.

    - countries
       Selected countries will be offered to the shop customer tho choose for
       shipping and invoice address.

    - default_country
       This country will be used to calculate shipping price if the shop
       customer doesn't have select a country yet.

    - checkout_type
       Decides whether the customer has to login, has not to login or has the
       choice to to login or not to be able to check out.
    """
    name = models.CharField(_(u"Name"), max_length=30)
    shop_owner = models.CharField(_(u"Shop owner"), max_length=100, blank=True)
    from_email = models.EmailField(_(u"From e-mail address"))
    notification_emails  = models.TextField(_(u"Notification email addresses"))

    description = models.TextField(_(u"Description"), blank=True)
    image = ImageWithThumbsField(_(u"Image"), upload_to="images", blank=True, null=True, sizes=((60, 60), (100, 100), (200, 200), (400, 400)))
    static_block = models.ForeignKey(StaticBlock, verbose_name=_(u"Static block"), blank=True, null=True, related_name="shops")

    product_cols = models.IntegerField(_(u"Product cols"), default=3)
    product_rows = models.IntegerField(_(u"Product rows"), default=3)
    category_cols = models.IntegerField(_(u"Category cols"), default=3)
    google_analytics_id = models.CharField(_(u"Google Analytics ID"), blank=True, max_length=20)
    ga_site_tracking = models.BooleanField(_(u"Google Analytics Site Tracking"), default=False)
    ga_ecommerce_tracking = models.BooleanField(_(u"Google Analytics E-Commerce Tracking"), default=False)

    countries = models.ManyToManyField(Country, verbose_name=_(u"Countries"), related_name="shops")
    default_country = models.ForeignKey(Country, verbose_name=_(u"Default country"))
    default_currency = models.CharField(_(u"Default Currency"), max_length=30, default="EUR")

    checkout_type = models.PositiveSmallIntegerField(_(u"Checkout type"), choices=CHECKOUT_TYPES, default=CHECKOUT_TYPE_SELECT)

    class Meta:
        permissions = (("manage_shop", "Manage shop"),)

    def __unicode__(self):
        return self.name

    def get_format_info(self):
        """
        """
        return {
            "product_cols" : self.product_cols,
            "product_rows" : self.product_rows,
            "category_cols" : self.category_cols,
        }

    def get_notification_emails(self):
        """Returns the notification e-mail addresses as list
        """
        adresses = re.split("[\s,]+", self.notification_emails)
        return adresses

    def get_parent_for_portlets(self):
        """Implements contract for django-portlets. Returns always None as there
        is no parent for a shop.
        """
        return None

from monkeys import *



from south.db import db
from django.db import models
from lfs.order.models import *

class Migration:
    
    def forwards(self, orm):
        
        # Adding field 'Order.invoice_company_name'
        db.add_column('order_order', 'invoice_company_name', models.CharField(_(u"Invoice company name"), max_length=50))
        
        # Adding field 'Order.shipping_company_name'
        db.add_column('order_order', 'shipping_company_name', models.CharField(_(u"Shipping company name"), max_length=50))
        
    
    
    def backwards(self, orm):
        
        # Deleting field 'Order.invoice_company_name'
        db.delete_column('order_order', 'invoice_company_name')
        
        # Deleting field 'Order.shipping_company_name'
        db.delete_column('order_order', 'shipping_company_name')
        
    
    
    models = {
        'payment.paymentmethod': {
            'Meta': {'ordering': '("priority",)'},
            '_stub': True,
            'id': ('models.AutoField', [], {'primary_key': 'True'})
        },
        'auth.user': {
            '_stub': True,
            'id': ('models.AutoField', [], {'primary_key': 'True'})
        },
        'shipping.shippingmethod': {
            'Meta': {'ordering': '("priority",)'},
            '_stub': True,
            'id': ('models.AutoField', [], {'primary_key': 'True'})
        },
        'catalog.product': {
            'Meta': {'ordering': '("name",)'},
            '_stub': True,
            'id': ('models.AutoField', [], {'primary_key': 'True'})
        },
        'order.order': {
            'Meta': {'ordering': '("-created",)'},
            'account_number': ('models.CharField', ['_(u"Account number")'], {'max_length': '30', 'blank': 'True'}),
            'bank_identification_code': ('models.CharField', ['_(u"Bank identication code")'], {'max_length': '30', 'blank': 'True'}),
            'bank_name': ('models.CharField', ['_(u"Bank name")'], {'max_length': '100', 'blank': 'True'}),
            'created': ('models.DateTimeField', ['_(u"Created")'], {'auto_now_add': 'True'}),
            'customer_email': ('models.CharField', ['_(u"lastname")'], {'max_length': '50'}),
            'customer_firstname': ('models.CharField', ['_(u"firstname")'], {'max_length': '50'}),
            'customer_lastname': ('models.CharField', ['_(u"lastname")'], {'max_length': '50'}),
            'depositor': ('models.CharField', ['_(u"Depositor")'], {'max_length': '100', 'blank': 'True'}),
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'invoice_city': ('models.CharField', ['_(u"Invoice city")'], {'max_length': '50'}),
            'invoice_company_name': ('models.CharField', ['_(u"Invoice company name")'], {'max_length': '50'}),
            'invoice_country': ('models.ForeignKey', ["orm['core.Country']"], {'related_name': '"orders_invoice_country"'}),
            'invoice_firstname': ('models.CharField', ['_(u"Invoice firstname")'], {'max_length': '50'}),
            'invoice_lastname': ('models.CharField', ['_(u"Invoice lastname")'], {'max_length': '50'}),
            'invoice_phone': ('models.CharField', ['_(u"Invoice phone")'], {'max_length': '20', 'blank': 'True'}),
            'invoice_state': ('models.CharField', ['_(u"Invoice state")'], {'max_length': '50'}),
            'invoice_street': ('models.CharField', ['_(u"Invoice street")'], {'max_length': '100', 'blank': 'True'}),
            'invoice_zip_code': ('models.CharField', ['_(u"Invoice zip code")'], {'max_length': '10'}),
            'message': ('models.TextField', ['_(u"Message")'], {'blank': 'True'}),
            'payment_method': ('models.ForeignKey', ["orm['payment.PaymentMethod']"], {'null': 'True', 'blank': 'True'}),
            'payment_price': ('models.FloatField', ['_(u"Payment Price")'], {'default': '0.0'}),
            'payment_tax': ('models.FloatField', ['_(u"Payment Tax")'], {'default': '0.0'}),
            'price': ('models.FloatField', ['_(u"Price")'], {'default': '0.0'}),
            'session': ('models.CharField', ['_(u"Session")'], {'max_length': '100', 'blank': 'True'}),
            'shipping_city': ('models.CharField', ['_(u"Shipping city")'], {'max_length': '50'}),
            'shipping_company_name': ('models.CharField', ['_(u"Shipping company name")'], {'max_length': '50'}),
            'shipping_country': ('models.ForeignKey', ["orm['core.Country']"], {'related_name': '"orders_shipping_country"'}),
            'shipping_firstname': ('models.CharField', ['_(u"Shipping firstname")'], {'max_length': '50'}),
            'shipping_lastname': ('models.CharField', ['_(u"Shipping lastname")'], {'max_length': '50'}),
            'shipping_method': ('models.ForeignKey', ["orm['shipping.ShippingMethod']"], {'null': 'True', 'blank': 'True'}),
            'shipping_phone': ('models.CharField', ['_(u"Shipping phone")'], {'max_length': '20', 'blank': 'True'}),
            'shipping_price': ('models.FloatField', ['_(u"Shipping Price")'], {'default': '0.0'}),
            'shipping_state': ('models.CharField', ['_(u"Shipping state")'], {'max_length': '50'}),
            'shipping_street': ('models.CharField', ['_(u"Shipping street")'], {'max_length': '100', 'blank': 'True'}),
            'shipping_tax': ('models.FloatField', ['_(u"Shipping Tax")'], {'default': '0.0'}),
            'shipping_zip_code': ('models.CharField', ['_(u"Shipping zip code")'], {'max_length': '10'}),
            'state': ('models.PositiveSmallIntegerField', ['_(u"State")'], {'default': '0'}),
            'tax': ('models.FloatField', ['_(u"Tax")'], {'default': '0.0'}),
            'user': ('models.ForeignKey', ["orm['auth.User']"], {'null': 'True', 'blank': 'True'}),
            'uuid': ('models.CharField', [], {'default': 'get_unique_id_str', 'unique': 'True', 'max_length': '50', 'editable': 'False'})
        },
        'order.orderitem': {
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'order': ('models.ForeignKey', ["orm['order.Order']"], {'related_name': '"items"'}),
            'price_gross': ('models.FloatField', ['_(u"Price gross")'], {'default': '0.0'}),
            'price_net': ('models.FloatField', ['_(u"Price net")'], {'default': '0.0'}),
            'product': ('models.ForeignKey', ["orm['catalog.Product']"], {'null': 'True', 'blank': 'True'}),
            'product_amount': ('models.IntegerField', ['_(u"Product quantity")'], {'null': 'True', 'blank': 'True'}),
            'product_name': ('models.CharField', ['_(u"Product name")'], {'max_length': '100', 'blank': 'True'}),
            'product_price_gross': ('models.FloatField', ['_(u"Product price gross")'], {'default': '0.0'}),
            'product_price_net': ('models.FloatField', ['_(u"Product price net")'], {'default': '0.0'}),
            'product_sku': ('models.CharField', ['_(u"Product SKU")'], {'max_length': '100', 'blank': 'True'}),
            'product_tax': ('models.FloatField', ['_(u"Product tax")'], {'default': '0.0'}),
            'tax': ('models.FloatField', ['_(u"Tax")'], {'default': '0.0'})
        },
        'core.country': {
            'Meta': {'ordering': '("name",)'},
            '_stub': True,
            'id': ('models.AutoField', [], {'primary_key': 'True'})
        }
    }
    
    complete_apps = ['order']




from south.db import db
from django.db import models
from lfs.criteria.models import *

class Migration:
    
    def forwards(self, orm):
        
        # Adding model 'CombinedLengthAndGirthCriterion'
        db.create_table('criteria_combinedlengthandgirthcriterion', (
            ('id', models.AutoField(primary_key=True)),
            ('operator', models.PositiveIntegerField(_(u"Operator"), null=True, blank=True)),
            ('clag', models.FloatField(_(u"Width"), default=0.0)),
        ))
        db.send_create_signal('criteria', ['CombinedLengthAndGirthCriterion'])
        
    def backwards(self, orm):
        
        # Deleting model 'CombinedLengthAndGirthCriterion'
        db.delete_table('criteria_combinedlengthandgirthcriterion')
    
    models = {
        'criteria.weightcriterion': {
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'operator': ('models.PositiveIntegerField', ['_(u"Operator")'], {'null': 'True', 'blank': 'True'}),
            'weight': ('models.FloatField', ['_(u"Weight")'], {'default': '0.0'})
        },
        'criteria.combinedlengthandgirthcriterion': {
            'clag': ('models.FloatField', ['_(u"Width")'], {'default': '0.0'}),
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'operator': ('models.PositiveIntegerField', ['_(u"Operator")'], {'null': 'True', 'blank': 'True'})
        },
        'criteria.heightcriterion': {
            'height': ('models.FloatField', ['_(u"Height")'], {'default': '0.0'}),
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'operator': ('models.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'})
        },
        'auth.user': {
            '_stub': True,
            'id': ('models.AutoField', [], {'primary_key': 'True'})
        },
        'criteria.criteriaobjects': {
            'Meta': {'ordering': '["position"]'},
            'content_id': ('models.PositiveIntegerField', ['_(u"Content id")'], {}),
            'content_type': ('models.ForeignKey', ["orm['contenttypes.ContentType']"], {'related_name': '"content_type"'}),
            'criterion_id': ('models.PositiveIntegerField', ['_(u"Content id")'], {}),
            'criterion_type': ('models.ForeignKey', ["orm['contenttypes.ContentType']"], {'related_name': '"criterion"'}),
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'position': ('models.PositiveIntegerField', ['_(u"Position")'], {'default': '999'})
        },
        'criteria.usercriterion': {
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'users': ('models.ManyToManyField', ["orm['auth.User']"], {})
        },
        'criteria.countrycriterion': {
            'countries': ('models.ManyToManyField', ["orm['core.Country']"], {}),
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'operator': ('models.PositiveIntegerField', ['_(u"Operator")'], {'null': 'True', 'blank': 'True'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label','model'),)", 'db_table': "'django_content_type'"},
            '_stub': True,
            'id': ('models.AutoField', [], {'primary_key': 'True'})
        },
        'criteria.lengthcriterion': {
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'length': ('models.FloatField', ['_(u"Length")'], {'default': '0.0'}),
            'operator': ('models.PositiveIntegerField', ['_(u"Operator")'], {'null': 'True', 'blank': 'True'})
        },
        'core.country': {
            'Meta': {'ordering': '("name",)'},
            '_stub': True,
            'id': ('models.AutoField', [], {'primary_key': 'True'})
        },
        'criteria.cartpricecriterion': {
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'operator': ('models.PositiveIntegerField', ['_(u"Operator")'], {'null': 'True', 'blank': 'True'}),
            'price': ('models.FloatField', ['_(u"Price")'], {'default': '0.0'})
        },
        'criteria.widthcriterion': {
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'operator': ('models.PositiveIntegerField', ['_(u"Operator")'], {'null': 'True', 'blank': 'True'}),
            'width': ('models.FloatField', ['_(u"Width")'], {'default': '0.0'})
        }
    }
    
    complete_apps = ['criteria']



# django imports
from django.contrib.auth.decorators import permission_required
from django.core.urlresolvers import reverse
from django.forms import ModelForm
from django.http import HttpResponseRedirect
from django.shortcuts import render_to_response
from django.shortcuts import get_object_or_404
from django.template import RequestContext
from django.utils.translation import ugettext_lazy as _

# lfs imports
import lfs.core.utils
from lfs.catalog.models import StaticBlock

class StaticBlockForm(ModelForm):
    """Form to add and edit a static block.
    """
    class Meta:
        model = StaticBlock

@permission_required("manage_shop", login_url="/login/")
def manage_static_blocks(request):
    """Dispatches to the first static block or to the add static block form.
    """
    try:
        sb = StaticBlock.objects.all()[0]
        url = reverse("lfs_manage_static_block", kwargs={"id": sb.id})
    except IndexError:
        url = reverse("lfs_add_static_block")
    
    return HttpResponseRedirect(url)

@permission_required("manage_shop", login_url="/login/")
def manage_static_block(request, id, template_name="manage/static_block/static_block.html"):
    """Displays the main form to manage static blocks.
    """
    sb = get_object_or_404(StaticBlock, pk=id)
    if request.method == "POST":
        form = StaticBlockForm(instance=sb, data=request.POST)
        if form.is_valid():
            form.save()
            return lfs.core.utils.set_message_cookie(
                url = reverse("lfs_manage_static_block", kwargs={"id" : sb.id}),
                msg = _(u"Static block has been saved."),
            )            
    else:
        form = StaticBlockForm(instance=sb)
        
    return render_to_response(template_name, RequestContext(request, {
        "static_block" : sb,
        "static_blocks" : StaticBlock.objects.all(),
        "form" : form,
        "current_id" : int(id),
    }))

@permission_required("manage_shop", login_url="/login/")    
def add_static_block(request, template_name="manage/static_block/add_static_block.html"):
    """Provides a form to add a new static block.
    """
    if request.method == "POST":
        form = StaticBlockForm(data=request.POST)
        if form.is_valid():
            new_sb = form.save()
            return lfs.core.utils.set_message_cookie(
                url = reverse("lfs_manage_static_block", kwargs={"id" : new_sb.id}),
                msg = _(u"Static block has been added."),
            )            
    else:
        form = StaticBlockForm()

    return render_to_response(template_name, RequestContext(request, {
        "form" : form,
        "static_blocks" : StaticBlock.objects.all(),        
    }))

@permission_required("manage_shop", login_url="/login/")
def preview_static_block(request, id, template_name="manage/static_block/preview.html"):
    """Displays a preview of an static block
    """
    sb = get_object_or_404(StaticBlock, pk=id)

    return render_to_response(template_name, RequestContext(request, {
        "static_block" : sb,        
    }))

@permission_required("manage_shop", login_url="/login/")
def delete_static_block(request, id):
    """Deletes static block with passed id.
    """
    sb = get_object_or_404(StaticBlock, pk=id)
    
    # First we delete all referencing categories. Otherwise they would be 
    # deleted
    for category in sb.categories.all():
        category.static_block = None
        category.save()    
    sb.delete()
    
    return lfs.core.utils.set_message_cookie(
        url = reverse("lfs_manage_static_blocks"),
        msg = _(u"Static block has been deleted."),
    )


import sys
import tempfile
import hotshot
import hotshot.stats
from django.conf import settings
from cStringIO import StringIO

from django.conf import settings
from django.http import HttpResponseServerError

class ProfileMiddleware(object):
    """
    Displays hotshot profiling for any view.
    http://yoursite.com/yourview/?prof

    Add the "prof" key to query string by appending ?prof (or &prof=)
    and you'll see the profiling results in your browser.
    It's set up to only be available in django's debug mode,
    but you really shouldn't add this middleware to any production configuration.
    * Only tested on Linux
    """
    def process_request(self, request):
        if request.GET.has_key('prof'):
            self.tmpfile = tempfile.NamedTemporaryFile()
            self.prof = hotshot.Profile(self.tmpfile.name)

    def process_view(self, request, callback, callback_args, callback_kwargs):
        if request.GET.has_key('prof'):
            return self.prof.runcall(callback, request, *callback_args, **callback_kwargs)

    def process_response(self, request, response):
        if request.GET.has_key('prof'):
            self.prof.close()

            out = StringIO()
            old_stdout = sys.stdout
            sys.stdout = out

            stats = hotshot.stats.load(self.tmpfile.name)
            # stats.strip_dirs()
            stats.sort_stats('cumulative', )
            # stats.sort_stats('time', )            
            stats.print_stats()

            sys.stdout = old_stdout
            stats_str = out.getvalue()

            if response and response.content and stats_str:
                response.content = "<pre>" + stats_str + "</pre>"

        return response
        
        
class AJAXSimpleExceptionResponse:
    def process_exception(self, request, exception):
        if settings.DEBUG:
            if request.is_ajax():
                import sys, traceback
                (exc_type, exc_info, tb) = sys.exc_info()
                response = "%s\n" % exc_type.__name__
                response += "%s\n\n" % exc_info
                response += "TRACEBACK:\n"    
                for tb in traceback.format_tb(tb):
                    response += "%s\n" % tb
                return HttpResponseServerError(response)



from south.db import db
from django.db import models
from lfs.payment.models import *

class Migration:
    
    def forwards(self, orm):
        
        # Adding model 'PayPalOrderTransaction'
        db.create_table('payment_paypalordertransaction', (
            ('id', models.AutoField(primary_key=True)),
            ('order', models.ForeignKey(orm['order.Order'], unique=True)),
        ))
        db.send_create_signal('payment', ['PayPalOrderTransaction'])
        
        # Adding ManyToManyField 'PayPalOrderTransaction.ipn'
        db.create_table('payment_paypalordertransaction_ipn', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('paypalordertransaction', models.ForeignKey(orm.PayPalOrderTransaction, null=False)),
            ('paypalipn', models.ForeignKey(orm['ipn.PayPalIPN'], null=False))
        ))
    
    def backwards(self, orm):
        
        # Deleting model 'PayPalOrderTransaction'
        db.delete_table('payment_paypalordertransaction')
        
        # Dropping ManyToManyField 'PayPalOrderTransaction.ipn'
        db.delete_table('payment_paypalordertransaction_ipn')
            
    models = {
        'ipn.paypalipn': {
            'Meta': {'db_table': '"paypal_ipn"'},
            '_stub': True,
            'id': ('models.AutoField', [], {'primary_key': 'True'})
        },
        'payment.paypalordertransaction': {
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'ipn': ('models.ManyToManyField', ["orm['ipn.PayPalIPN']"], {}),
            'order': ('models.ForeignKey', ["orm['order.Order']"], {'unique': 'True'})
        },
        'payment.paymentmethod': {
            'Meta': {'ordering': '("priority",)'},
            'active': ('models.BooleanField', ['_(u"Active")'], {'default': 'False'}),
            'criteria_objects': ('generic.GenericRelation', ["orm['criteria.CriteriaObjects']"], {'object_id_field': '"content_id"', 'content_type_field': '"content_type"'}),
            'deletable': ('models.BooleanField', [], {'default': 'True'}),
            'description': ('models.TextField', ['_(u"Description")'], {'blank': 'True'}),
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'image': ('models.ImageField', ['_(u"Image")'], {'null': 'True', 'blank': 'True'}),
            'name': ('models.CharField', ['_(u"Name")'], {'max_length': '50', 'blank': 'True'}),
            'note': ('models.TextField', ['_(u"note")'], {'blank': 'True'}),
            'price': ('models.FloatField', ['_(u"Price")'], {'default': '0.0'}),
            'priority': ('models.IntegerField', ['_(u"Priority")'], {'default': '0'}),
            'tax': ('models.ForeignKey', ["orm['tax.Tax']"], {'null': 'True', 'blank': 'True'})
        },
        'order.order': {
            'Meta': {'ordering': '("-created",)'},
            '_stub': True,
            'id': ('models.AutoField', [], {'primary_key': 'True'})
        },
        'tax.tax': {
            '_stub': True,
            'id': ('models.AutoField', [], {'primary_key': 'True'})
        },
        'criteria.criteriaobjects': {
            '_stub': True,
            'id': ('models.AutoField', [], {'primary_key': 'True'})
        },
        'payment.paymentmethodprice': {
            'Meta': {'ordering': '("priority",)'},
            'active': ('models.BooleanField', ['_(u"Active")'], {'default': 'False'}),
            'criteria_objects': ('generic.GenericRelation', ["orm['criteria.CriteriaObjects']"], {'object_id_field': '"content_id"', 'content_type_field': '"content_type"'}),
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'payment_method': ('models.ForeignKey', ["orm['payment.PaymentMethod']"], {'related_name': '"prices"'}),
            'price': ('models.FloatField', ['_(u"Price")'], {'default': '0.0'}),
            'priority': ('models.IntegerField', ['_(u"Priority")'], {'default': '0'})
        }
    }
    
    complete_apps = ['payment']



# lfs imports
from lfs.core.signals import customer_added
from lfs.core.signals import order_submitted
from lfs.mail import utils as mail_utils

# reviews imports
from reviews.signals import review_added

def order_submitted_listener(sender, **kwargs):
    """Listen to order submitted signal
    """
    mail_utils.send_order_received_mail(sender)
order_submitted.connect(order_submitted_listener)

def customer_added_listener(sender, **kwargs):
    """Listens to customer added signal.
    """
    mail_utils.send_customer_added(sender)
customer_added.connect(customer_added_listener)

def review_added_listener(sender, **kwargs):
    """Listens to review added signal
    """
    mail_utils.send_review_added(sender)
review_added.connect(review_added_listener)



from south.db import db
from django.db import models
from lfs.page.models import *

class Migration:
    
    def forwards(self, orm):
        
        # Adding model 'Page'
        db.create_table('page_page', (
            ('body', models.TextField(_(u"Text"), blank=True)),
            ('title', models.CharField(_(u"Title"), max_length=100)),
            ('id', models.AutoField(primary_key=True)),
            ('file', models.FileField(_(u"File"), upload_to="files", blank=True)),
            ('active', models.BooleanField(_(u"Active"), default=False)),
            ('position', models.IntegerField(_(u"Position"), default=999)),
            ('short_text', models.TextField(blank=True)),
            ('slug', models.CharField(_(u"Slug"), max_length=100)),
        ))
        db.send_create_signal('page', ['Page'])
        
    
    
    def backwards(self, orm):
        
        # Deleting model 'Page'
        db.delete_table('page_page')
        
    
    
    models = {
        'page.page': {
            'Meta': {'ordering': '("position",)'},
            'active': ('models.BooleanField', ['_(u"Active")'], {'default': 'False'}),
            'body': ('models.TextField', ['_(u"Text")'], {'blank': 'True'}),
            'file': ('models.FileField', ['_(u"File")'], {'upload_to': '"files"', 'blank': 'True'}),
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'position': ('models.IntegerField', ['_(u"Position")'], {'default': '999'}),
            'short_text': ('models.TextField', [], {'blank': 'True'}),
            'slug': ('models.CharField', ['_(u"Slug")'], {'max_length': '100'}),
            'title': ('models.CharField', ['_(u"Title")'], {'max_length': '100'})
        }
    }
    
    complete_apps = ['page']




from south.db import db
from django.db import models
from lfs.core.models import *

class Migration:
    
    def forwards(self, orm):
        
        # Adding field 'Shop.default_currency'
        db.add_column('core_shop', 'default_currency', models.CharField(_(u"Default Currency"), default='EUR', max_length=30))
        
    
    
    def backwards(self, orm):
        
        # Deleting field 'Shop.default_currency'
        db.delete_column('core_shop', 'default_currency')
        
    
    
    models = {
        'core.action': {
            'Meta': {'ordering': '("position",)'},
            'active': ('models.BooleanField', ['_(u"Active")'], {'default': 'False'}),
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'link': ('models.CharField', ['_(u"Link")'], {'max_length': '100', 'blank': 'True'}),
            'parent': ('models.ForeignKey', ["orm['core.Action']"], {'null': 'True', 'blank': 'True'}),
            'place': ('models.PositiveSmallIntegerField', [], {'default': '1', 'null': 'True', 'blank': 'True'}),
            'position': ('models.IntegerField', ['_(u"Position")'], {'default': '999'}),
            'title': ('models.CharField', ['_(u"Title")'], {'max_length': '40'})
        },
        'catalog.staticblock': {
            '_stub': True,
            'id': ('models.AutoField', [], {'primary_key': 'True'})
        },
        'core.shop': {
            'Meta': {'permissions': '(("manage_shop","Manage shop"),)'},
            'category_cols': ('models.IntegerField', ['_(u"Category cols")'], {'default': '3'}),
            'checkout_type': ('models.PositiveSmallIntegerField', ['_(u"Checkout type")'], {'default': '0'}),
            'countries': ('models.ManyToManyField', ["orm['core.Country']"], {'related_name': '"shops"'}),
            'default_country': ('models.ForeignKey', ["orm['core.Country']"], {}),
            'default_currency': ('models.CharField', ['_(u"Default Currency")'], {'default': "'EUR'", 'max_length': '30'}),
            'description': ('models.TextField', ['_(u"Description")'], {'blank': 'True'}),
            'from_email': ('models.EmailField', ['_(u"From e-mail address")'], {}),
            'ga_ecommerce_tracking': ('models.BooleanField', ['_(u"Google Analytics E-Commerce Tracking")'], {'default': 'False'}),
            'ga_site_tracking': ('models.BooleanField', ['_(u"Google Analytics Site Tracking")'], {'default': 'False'}),
            'google_analytics_id': ('models.CharField', ['_(u"Google Analytics ID")'], {'max_length': '20', 'blank': 'True'}),
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'image': ('ImageWithThumbsField', ['_(u"Image")'], {'null': 'True', 'sizes': '((60,60),(100,100),(200,200),(400,400))', 'blank': 'True'}),
            'name': ('models.CharField', ['_(u"Name")'], {'max_length': '30'}),
            'notification_emails': ('models.TextField', ['_(u"Notification email addresses")'], {}),
            'product_cols': ('models.IntegerField', ['_(u"Product cols")'], {'default': '3'}),
            'product_rows': ('models.IntegerField', ['_(u"Product rows")'], {'default': '3'}),
            'shop_owner': ('models.CharField', ['_(u"Shop owner")'], {'max_length': '100', 'blank': 'True'}),
            'static_block': ('models.ForeignKey', ["orm['catalog.StaticBlock']"], {'related_name': '"shops"', 'null': 'True', 'blank': 'True'})
        },
        'core.country': {
            'Meta': {'ordering': '("name",)'},
            'code': ('models.CharField', ['_(u"Country code")'], {'max_length': '2'}),
            'id': ('models.AutoField', [], {'primary_key': 'True'}),
            'name': ('models.CharField', ['_(u"Name")'], {'max_length': '100'})
        }
    }
    
    complete_apps = ['core']



# django imports
from django.contrib.auth.decorators import permission_required
from django.core.urlresolvers import reverse
from django.forms import ModelForm
from django.http import HttpResponse
from django.http import HttpResponseRedirect
from django.shortcuts import render_to_response
from django.shortcuts import get_object_or_404
from django.template import RequestContext
from django.template.loader import render_to_string
from django.utils import simplejson
from django.utils.translation import ugettext_lazy as _

# lfs imports
import lfs.core.utils
from lfs.core.utils import LazyEncoder
from lfs.core.signals import property_type_changed
from lfs.catalog.models import Property
from lfs.catalog.models import PropertyOption
from lfs.catalog.models import FilterStep

class PropertyDataForm(ModelForm):
    """Form to manage core data of a property.
    """
    class Meta:
        model = Property
        fields = ["name", "filterable", "unit", "position", "display_on_product", "display_no_results"]

class PropertyTypeForm(ModelForm):
    """Form to manage property type.
    """
    class Meta:
        model = Property
        fields = ["type"]

class StepTypeForm(ModelForm):
    """Form to manage the step type of a property.
    """
    class Meta:
        model = Property
        fields = ["step_type"]

class StepForm(ModelForm):
    """Form to manage step range.
    """
    class Meta:
        model = Property
        fields = ["step"]

@permission_required("manage_shop", login_url="/login/")    
def manage_properties(request):
    """The main view to manage properties.
    """
    try:
        property = Property.objects.filter(local=False)[0]
        url = reverse("lfs_manage_shop_property", kwargs={"id": property.id})
    except IndexError:
        url = reverse("lfs_add_shop_property")
    
    return HttpResponseRedirect(url)

@permission_required("manage_shop", login_url="/login/")    
def manage_property(request, id, template_name="manage/properties/property.html"):
    """
    """    
    property = get_object_or_404(Property, pk=id)
    if request.method == "POST":
        form = PropertyDataForm(instance=property, data=request.POST)
        if form.is_valid():
            new_property = form.save()

            return lfs.core.utils.set_message_cookie(
                url = reverse("lfs_manage_shop_property", kwargs={"id" : property.id}),
                msg = _(u"Property type has been saved."),
            )        
            
    else:
        form = PropertyDataForm(instance=property)
    
    return render_to_response(template_name, RequestContext(request, {
        "property" : property,
        "properties" : Property.objects.filter(local=False),
        "form" : form,
        "type_form" : PropertyTypeForm(instance=property),
        "current_id" : int(id),
        "options" : options_inline(request, id),
        "steps" : steps_inline(request, id),
    }))

@permission_required("manage_shop", login_url="/login/")    
def update_property_type(request, id):
    """Updates the type of the property.
    
    This is separated from the data, because a change of type causes a deletion
    of product property values
    """
    property = get_object_or_404(Property, pk=id)
    old_type = property.type
    form = PropertyTypeForm(instance=property, data=request.POST)
    new_property = form.save()
        
    # Send signal only when the type changed as all values are deleted.
    if old_type != new_property.type:
        property_type_changed.send(property)

    return lfs.core.utils.set_message_cookie(
        url = reverse("lfs_manage_shop_property", kwargs={"id" : property.id}),
        msg = _(u"Property type has been changed."),
    )        

@permission_required("manage_shop", login_url="/login/")
def steps_inline(request, property_id, template_name="manage/properties/step_inline.html"):
    """Display the steps of a propety. Factored out for Ajax requests.
    """
    property = get_object_or_404(Property, pk=property_id)

    step_form = StepForm(instance=property)    
    step_type_form = StepTypeForm(instance=property)
    
    return render_to_string(template_name, RequestContext(request, {
        "property" : property,
        "step_form" : step_form,
        "step_type_form" : step_type_form,
    }))

def save_step(request, property_id):
    """Save the steps of the property with given id.
    """    
    property = get_object_or_404(Property, pk=property_id)
    
    form = StepForm(instance=property, data=request.POST)
    property = form.save()

    result = simplejson.dumps({
        "steps" : steps_inline(request, property_id),
        "message" : _(u"Steps have been saved."),
    }, cls = LazyEncoder)
    
    return HttpResponse(result)

def save_step_type(request, property_id):
    """Save the step type of the property with given id.
    """    
    property = get_object_or_404(Property, pk=property_id)
    
    form = StepTypeForm(instance=property, data=request.POST)
    property = form.save()

    result = simplejson.dumps({
        "steps" : steps_inline(request, property_id),
        "message" : _(u"Step type has been saved."),
    }, cls = LazyEncoder)
    
    return HttpResponse(result)

@permission_required("manage_shop", login_url="/login/")
def add_step(request, property_id):
    """Adds a step to property with passed property id resp. updates steps of 
    property with passed property id dependent on the given action parameter.
    """
    property = get_object_or_404(Property, pk=property_id)
    
    if request.POST.get("action") == "add":
        start = request.POST.get("start", "")
        if start != "":
            option = FilterStep.objects.create(start=start, property_id=property_id)
        message = _(u"Step has been added.")
    else:
        
        for step_id in request.POST.getlist("step"):
            
            try:
                step = FilterStep.objects.get(pk=step_id)
            except FilterStep.DoesNotExist:
                pass
            else:
                step.start = request.POST.get("start-%s" % step_id, "") 
                step.save()
        message = _(u"Steps have been updated.")

    result = simplejson.dumps({
        "steps" : steps_inline(request, property_id),
        "message" : message
    }, cls = LazyEncoder)
    
    return HttpResponse(result)

@permission_required("manage_shop", login_url="/login/")    
def delete_step(request, id):
    """Deletes step with given id.
    """
    try:        
        step = FilterStep.objects.get(pk=id)
    except FilterStep.DoesNotExist:
        url = request.META.get("HTTP_REFERER", reverse("lfs_manage_shop_property"))
    else:
        property = step.property
        url = reverse("lfs_manage_shop_property", kwargs={"id" : property.id})
        step.delete()        
    
    return HttpResponseRedirect(url)

@permission_required("manage_shop", login_url="/login/")
def options_inline(request, property_id, template_name="manage/properties/options_inline.html"):
    """Display the options of a propety. Factored out for Ajax requests.
    """
    property = get_object_or_404(Property, pk=property_id)
    return render_to_string(template_name, RequestContext(request, {
        "property" : property,
    }))
    
@permission_required("manage_shop", login_url="/login/")
def add_property(request, template_name="manage/properties/add_property.html"):
    """Adds a new property.
    """
    if request.method == "POST":
        form = PropertyDataForm(data=request.POST)
        if form.is_valid():
            property = form.save()

            return lfs.core.utils.set_message_cookie(
                url = reverse("lfs_manage_shop_property", kwargs={"id" : property.id}),
                msg = _(u"Property has been added."),
            )        
    else:
        form = PropertyDataForm()

    return render_to_response(template_name, RequestContext(request, {
        "form" : form,
        "properties" : Property.objects.filter(local=False),
    }))

@permission_required("manage_shop", login_url="/login/")
def delete_property(request, id):
    """Deletes the property with given id.
    """
    try:
        property = Property.objects.get(pk=id)
    except Property.DoesNotExist:
        url = request.META.get("HTTP_REFERER", reverse("lfs_manage_shop_property"))
    else:
        property.delete()        
        url = reverse("lfs_manage_shop_properties")
        
    return HttpResponseRedirect(url)

@permission_required("manage_shop", login_url="/login/")
def add_option(request, property_id):
    """Adds option to property with passed property id.
    """
    property = get_object_or_404(Property, pk=property_id)
    
    if request.POST.get("action") == "add":
        name = request.POST.get("name", "")
        if name != "":
            option = PropertyOption.objects.create(name=name, property_id=property_id)
        message = _(u"Option has been added.")
    else:
        
        for option_id in request.POST.getlist("option"):
            
            try:
                option = PropertyOption.objects.get(pk=option_id)
            except PropertyOption.DoesNotExist:
                pass
            else:
                option.position = request.POST.get("position-%s" % option_id, 99)
                option.name = request.POST.get("name-%s" % option_id, "") 
                option.save()
        message = _(u"Options have been update.")

    _update_positions(property)
    result = simplejson.dumps({
        "options" : options_inline(request, property_id),
        "message" : message
    }, cls = LazyEncoder)
    
    return HttpResponse(result)

@permission_required("manage_shop", login_url="/login/")    
def delete_option(request, id):
    """Deletes option with given id.
    """
    try:        
        option = PropertyOption.objects.get(pk=id)
    except option.DoesNotExist:
        url = request.META.get("HTTP_REFERER", reverse("lfs_manage_shop_property"))
    else:
        property = option.property
        url = reverse("lfs_manage_shop_property", kwargs={"id" : property.id})
        option.delete()        
        _update_positions(property)
    
    return HttpResponseRedirect(url)
    
def _update_positions(property):
    """Updates position of options of given property.
    """
    for i, option in enumerate(property.options.all()):
        option.position = i+1
        option.save()



from django.utils.translation import gettext_lazy as _

ACTIVE_FOR_SALE_STANDARD = 0
ACTIVE_FOR_SALE_YES = 2
ACTIVE_FOR_SALE_NO = 3
ACTIVE_FOR_SALE_CHOICES = [
    (ACTIVE_FOR_SALE_STANDARD, _(u"Standard")),
    (ACTIVE_FOR_SALE_YES, _(u"Yes")),
    (ACTIVE_FOR_SALE_NO, _(u"No")),    
]

STANDARD_PRODUCT = "0"
PRODUCT_WITH_VARIANTS = "1"
VARIANT = "2"

PRODUCT_TYPE_LOOKUP = {
    STANDARD_PRODUCT : _(u"Standard"),
    PRODUCT_WITH_VARIANTS : _(u"Product with variants"),
    VARIANT : _(u"Variant"),
}

PRODUCT_TYPE_CHOICES = [
    (STANDARD_PRODUCT, _(u"Standard")),
    (PRODUCT_WITH_VARIANTS, _(u"Product with variants")),
    (VARIANT, _(u"Variant")),
]

PRODUCT_TYPE_FORM_CHOICES = [
    (STANDARD_PRODUCT, _(u"Standard")),
    (PRODUCT_WITH_VARIANTS, _(u"Product with variants")),
]


LIST = 0
SELECT = 1
VARIANTS_DISPLAY_TYPE_CHOICES = [
    (LIST, _(u"List")),
    (SELECT, _(u"Select")),
]

CONTENT_PRODUCTS = 1
CONTENT_CATEGORIES = 2

CONTENT_CHOICES = (
    (CONTENT_PRODUCTS, _(u"Products")),
    (CONTENT_CATEGORIES, _(u"Categories")),
)

DELIVERY_TIME_UNIT_HOURS  = 1
DELIVERY_TIME_UNIT_DAYS   = 2
DELIVERY_TIME_UNIT_WEEKS  = 3
DELIVERY_TIME_UNIT_MONTHS = 4

DELIVERY_TIME_UNIT_CHOICES = (
    (DELIVERY_TIME_UNIT_HOURS, _(u"hours")),
    (DELIVERY_TIME_UNIT_DAYS, _(u"days")),
    (DELIVERY_TIME_UNIT_WEEKS, _(u"weeks")),
    (DELIVERY_TIME_UNIT_MONTHS, _(u"months")),
)

DELIVERY_TIME_UNIT_SINGULAR = {
    DELIVERY_TIME_UNIT_HOURS : _(u"hour"),
    DELIVERY_TIME_UNIT_DAYS : _(u"day"),
    DELIVERY_TIME_UNIT_WEEKS : _(u"week"),
    DELIVERY_TIME_UNIT_MONTHS : _(u"month"),
}

PROPERTY_NUMBER_FIELD = 1
PROPERTY_TEXT_FIELD = 2
PROPERTY_SELECT_FIELD = 3

PROPERTY_TYPE_CHOICES = (
    (PROPERTY_NUMBER_FIELD, _(u"Number field")),
    (PROPERTY_TEXT_FIELD, _(u"Text field")),
    (PROPERTY_SELECT_FIELD, _(u"Select field")),
)

PROPERTY_STEP_TYPE_AUTOMATIC  = 1
PROPERTY_STEP_TYPE_FIXED_STEP = 2
PROPERTY_STEP_TYPE_MANUAL_STEPS = 3

PROPERTY_STEP_TYPE_CHOICES = (
    (PROPERTY_STEP_TYPE_AUTOMATIC,    _(u"Automatic")),
    (PROPERTY_STEP_TYPE_FIXED_STEP,   _(u"Fixed step")),
    (PROPERTY_STEP_TYPE_MANUAL_STEPS, _(u"Manual steps")),
)